@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -132,6 +132,15 @@ extern "C"
  */
   void eXosip_automatic_action(void);
 
+#ifndef MINISIZE
+  /**
+ * Automatic internal handling of known package.
+ * 
+ * @param evt          event related to an incoming subscription.
+ */
+  int eXosip_insubscription_automatic(eXosip_event_t *evt);
+#endif
+
 /**
  * Generate random string:
  *
@@ -163,6 +172,13 @@ extern "C"
  */
   sdp_message_t *eXosip_get_local_sdp(int did);
 
+/**
+ * Get local SDP body for the previous latest INVITE of call.
+ * 
+ * @param did          dialog id of call.
+ */
+  sdp_message_t *eXosip_get_previous_local_sdp (int did);
+
 /**
  * Get remote SDP body for the latest INVITE of call.
  * 
@@ -309,9 +325,7 @@ extern "C"
   struct eXosip_event
   {
     eXosip_event_type_t type;               /**< type of the event */
-#if !defined(_WIN32_WCE)
     char                textinfo[256];      /**< text description of event */ 
-#endif
     void               *external_reference; /**< external reference (for calls) */
     
     osip_message_t     *request;   /**< request within current transaction */
