@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -59,7 +59,11 @@ eXosip_create_transaction (eXosip_call_t * jc,
   sipevent = osip_new_outgoing_sipmessage (request);
   sipevent->transactionid = tr->transactionid;
 
+#ifndef MINISIZE
   osip_transaction_set_your_instance (tr, __eXosip_new_jinfo (jc, jd, NULL, NULL));
+#else
+  osip_transaction_set_your_instance (tr, __eXosip_new_jinfo (jc, jd));
+#endif
   osip_transaction_add_event (tr, sipevent);
   __eXosip_wakeup ();
   return 0;
@@ -278,8 +282,13 @@ eXosip_call_send_initial_invite (osip_message_t * invite)
   sipevent = osip_new_outgoing_sipmessage (invite);
   sipevent->transactionid = transaction->transactionid;
 
+#ifndef MINISIZE
   osip_transaction_set_your_instance (transaction,
                                       __eXosip_new_jinfo (jc, NULL, NULL, NULL));
+#else
+  osip_transaction_set_your_instance (transaction,
+                                      __eXosip_new_jinfo (jc, NULL));
+#endif
   osip_transaction_add_event (transaction, sipevent);
 
   jc->external_reference = NULL;
@@ -440,10 +449,19 @@ eXosip_call_send_ack (int did, osip_message_t * ack)
       host = route->url->host;
   } else
     {
+      /* search for maddr parameter */
+      osip_uri_param_t *maddr_param = NULL;
+      osip_uri_uparam_get_byname (ack->req_uri, "maddr", &maddr_param);
+      host=NULL;
+      if (maddr_param!=NULL && maddr_param->gvalue!=NULL)
+	host = maddr_param->gvalue;
+      
       port = 5060;
       if (ack->req_uri->port != NULL)
-        port = osip_atoi (ack->req_uri->port);
-      host = ack->req_uri->host;
+	port = osip_atoi (ack->req_uri->port);
+      
+      if (host==NULL)
+	host = ack->req_uri->host;
     }
 
   cb_snd_message (NULL, ack, host, port, -1);
@@ -526,6 +544,8 @@ eXosip_call_build_request (int jid, const char *method, osip_message_t ** reques
 
   if (jc->response_auth != NULL)
     eXosip_add_authentication_information (*request, jc->response_auth);
+  else
+	eXosip_add_authentication_information(*request, NULL);
 
   return 0;
 }
@@ -617,13 +637,20 @@ eXosip_call_send_request (int jid, osip_message_t * request)
   sipevent = osip_new_outgoing_sipmessage (request);
   sipevent->transactionid = transaction->transactionid;
 
+#ifndef MINISIZE
   osip_transaction_set_your_instance (transaction,
                                       __eXosip_new_jinfo (jc, jd, NULL, NULL));
+#else
+  osip_transaction_set_your_instance (transaction,
+                                      __eXosip_new_jinfo (jc, jd));
+#endif
   osip_transaction_add_event (transaction, sipevent);
   __eXosip_wakeup ();
   return 0;
 }
 
+#ifndef MINISIZE
+
 int
 eXosip_call_build_refer (int did, const char *refer_to, osip_message_t ** request)
 {
@@ -722,6 +749,8 @@ eXosip_call_build_notify (int did, int subscription_status,
   return 0;
 }
 
+#endif
+
 int
 eXosip_call_build_answer (int tid, int status, osip_message_t ** answer)
 {
@@ -746,15 +775,9 @@ eXosip_call_build_answer (int tid, int status, osip_message_t ** answer)
 
   if (0 == osip_strcasecmp (tr->orig_request->sip_method, "INVITE"))
     {
-      if (status > 100 && status < 200)
+      if (status > 100 && status <= 699)
         {
-          i = _eXosip_answer_invite_1xx (jc, jd, status, answer);
-      } else if (status > 199 && status < 300)
-        {
-          i = _eXosip_answer_invite_2xx (jc, jd, status, answer);
-      } else if (status > 300 && status <= 699)
-        {
-          i = _eXosip_answer_invite_3456xx (jc, jd, status, answer);
+          i = _eXosip_answer_invite_123456xx (jc, jd, status, answer, 0);
       } else
         {
           OSIP_TRACE (osip_trace
@@ -792,7 +815,6 @@ eXosip_call_build_answer (int tid, int status, osip_message_t ** answer)
 int
 eXosip_call_send_answer (int tid, int status, osip_message_t * answer)
 {
-  int i = -1;
   eXosip_dialog_t *jd = NULL;
   eXosip_call_t *jc = NULL;
   osip_transaction_t *tr = NULL;
@@ -816,16 +838,11 @@ eXosip_call_send_answer (int tid, int status, osip_message_t * answer)
     {
       if (0 == osip_strcasecmp (tr->orig_request->sip_method, "INVITE"))
         {
-          if (status >= 100 && status <= 199)
-            {
-          } else if (status >= 300 && status <= 699)
-            {
-          } else
+          if (status >= 200 && status <= 299)
             {
               OSIP_TRACE (osip_trace
                           (__FILE__, __LINE__, OSIP_ERROR, NULL,
                            "eXosip: Wrong parameter?\n"));
-              osip_message_free (answer);
               return -1;
             }
         }
@@ -848,86 +865,23 @@ eXosip_call_send_answer (int tid, int status, osip_message_t * answer)
     {
       if (0 == osip_strcasecmp (tr->orig_request->sip_method, "INVITE"))
         {
-          if (status < 200)
-            i = _eXosip_default_answer_invite_1xx (jc, jd, status);
-          else
-            i = _eXosip_default_answer_invite_3456xx (jc, jd, status);
-          if (i != 0)
-            {
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: cannot send response!\n"));
-              return -1;
-            }
-      } else
-        {
-          /* TODO */
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: a response must be given!\n"));
-          return -1;
-        }
-      return 0;
-  } else
-    {
-      i = 0;
-    }
-
+	  osip_message_t *response;
+          return _eXosip_answer_invite_123456xx (jc, jd, status, &response, 1);
+	}
+      return -1;
+  }
+  
   if (0 == osip_strcasecmp (tr->orig_request->sip_method, "INVITE"))
     {
-      if (MSG_IS_STATUS_1XX (answer))
-        {
-          if (jd == NULL)
-            {
-              i = eXosip_dialog_init_as_uas (&jd, tr->orig_request, answer);
-              if (i != 0)
-                {
-                  OSIP_TRACE (osip_trace
-                              (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                               "eXosip: cannot create dialog!\n"));
-                  i = 0;
-              } else
-                {
-                  ADD_ELEMENT (jc->c_dialogs, jd);
-                }
-            }
-      } else if (MSG_IS_STATUS_2XX (answer))
-        {
-          if (jd == NULL)
-            {
-              i = eXosip_dialog_init_as_uas (&jd, tr->orig_request, answer);
-              if (i != 0)
-                {
-                  OSIP_TRACE (osip_trace
-                              (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                               "eXosip: cannot create dialog!\n"));
-                  osip_message_free (answer);
-                  return -1;
-                }
-              ADD_ELEMENT (jc->c_dialogs, jd);
-          } else
-            i = 0;
-
-          eXosip_dialog_set_200ok (jd, answer);
-		  /* wait for a ACK */
-
-          osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
-      } else if (answer->status_code >= 300 && answer->status_code <= 699)
-        {
-          i = 0;
-      } else
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: wrong status code (101<status<=699)\n"));
-          osip_message_free (answer);
-          return -1;
-        }
-      if (i != 0)
-        {
-          osip_message_free (answer);
-          return -1;
-        }
+      if (MSG_IS_STATUS_2XX (answer) && jd!=NULL)
+	{
+	  if (status >= 200 && status <300 && jd!=NULL)
+	    {
+	      eXosip_dialog_set_200ok (jd, answer);
+	      /* wait for a ACK */
+	      osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
+	    }
+	}
     }
 
   evt_answer = osip_new_outgoing_sipmessage (answer);
@@ -1020,6 +974,8 @@ eXosip_call_terminate (int cid, int did)
 
           if (jc->response_auth != NULL)
             eXosip_add_authentication_information (request, jc->response_auth);
+		  else
+            eXosip_add_authentication_information (request, NULL);
 
           i = eXosip_create_transaction (jc, jd, request);
           if (i != 0)
@@ -1049,8 +1005,13 @@ eXosip_call_terminate (int cid, int did)
       if (tr != NULL && tr->last_response != NULL &&
           MSG_IS_STATUS_1XX (tr->last_response))
         {                       /* answer with 603 */
+		  osip_generic_param_t *to_tag;
+		  osip_from_param_get_byname (tr->orig_request->to, "tag", &to_tag);
+
           i = eXosip_call_send_answer (tr->transactionid, 603, NULL);
-          return i;
+
+		  if (to_tag==NULL)
+			  return i;
         }
     }
 
@@ -1071,6 +1032,8 @@ eXosip_call_terminate (int cid, int did)
     }
   if (jc->response_auth != NULL)
     eXosip_add_authentication_information (request, jc->response_auth);
+  else
+    eXosip_add_authentication_information (request, NULL);
 
   i = eXosip_create_transaction (jc, jd, request);
   if (i != 0)
@@ -1087,6 +1050,8 @@ eXosip_call_terminate (int cid, int did)
   return 0;
 }
 
+#ifndef MINISIZE
+
 int
 eXosip_call_build_prack (int tid, osip_message_t ** prack)
 {
@@ -1209,15 +1174,20 @@ eXosip_call_send_prack (int tid, osip_message_t * prack)
   sipevent = osip_new_outgoing_sipmessage (prack);
   sipevent->transactionid = tr->transactionid;
 
+#ifndef MINISIZE
   osip_transaction_set_your_instance (tr, __eXosip_new_jinfo (jc, jd, NULL, NULL));
+#else
+  osip_transaction_set_your_instance (tr, __eXosip_new_jinfo (jc, jd));
+#endif
   osip_transaction_add_event (tr, sipevent);
   __eXosip_wakeup ();
   return 0;
 }
 
+#endif
 
 int
-_eXosip_call_redirect_request (eXosip_call_t * jc,
+_eXosip_call_retry_request (eXosip_call_t * jc,
                                eXosip_dialog_t * jd, osip_transaction_t * out_tr)
 {
   osip_transaction_t *tr = NULL;
@@ -1261,216 +1231,123 @@ _eXosip_call_redirect_request (eXosip_call_t * jc,
       return -1;
     }
 
-  co = NULL;
-  pos = 0;
-  while (!osip_list_eol (&out_tr->last_response->contacts, pos))
+  if (MSG_IS_STATUS_3XX(out_tr->last_response))
     {
-      co = (osip_contact_t *) osip_list_get (&out_tr->last_response->contacts, pos);
-      if (co != NULL && co->url != NULL)
-        {
-          /* check tranport? Only allow UDP, right now */
-          osip_uri_param_t *u_param;
-          int pos2;
-
-          u_param = NULL;
-          pos2 = 0;
-          while (!osip_list_eol (&co->url->url_params, pos2))
-            {
-              u_param =
-                (osip_uri_param_t *) osip_list_get (&co->url->url_params, pos2);
-              if (u_param == NULL || u_param->gname == NULL
-                  || u_param->gvalue == NULL)
-                {
-                  u_param = NULL;
-                  /* skip */
-              } else if (0 == osip_strcasecmp (u_param->gvalue, "transport"))
-                {
-                  if (0 == osip_strcasecmp (u_param->gvalue, "udp"))
-                    {
-#if 0
-                      /* remove the UDP parameter */
-                      osip_list_remove (co->url->url_params, pos2);
-                      osip_uri_param_free (u_param);
-#endif
-                      u_param = NULL;
-                      protocol = IPPROTO_UDP;
-                      break;    /* ok */
-                  } else if (0 == osip_strcasecmp (u_param->gvalue, "tcp"))
-                    {
-#if 0
-                      osip_list_remove (co->url->url_params, pos2);
-                      osip_uri_param_free (u_param);
-#endif
-                      protocol = IPPROTO_TCP;
-                      u_param = NULL;
-                    }
-                  break;
-                }
-              pos2++;
-            }
-
-          if (u_param == NULL || u_param->gname == NULL || u_param->gvalue == NULL)
-            {
-              break;            /* default is udp! */
-            }
-        }
-      pos++;
       co = NULL;
+      pos = 0;
+      while (!osip_list_eol (&out_tr->last_response->contacts, pos))
+	{
+	  co = (osip_contact_t *) osip_list_get (&out_tr->last_response->contacts, pos);
+	  if (co != NULL && co->url != NULL)
+	    {
+	      /* check tranport? Only allow UDP, right now */
+	      osip_uri_param_t *u_param;
+	      int pos2;
+	      
+	      u_param = NULL;
+	      pos2 = 0;
+	      while (!osip_list_eol (&co->url->url_params, pos2))
+		{
+		  u_param =
+		    (osip_uri_param_t *) osip_list_get (&co->url->url_params, pos2);
+		  if (u_param == NULL || u_param->gname == NULL
+		      || u_param->gvalue == NULL)
+		    {
+		      u_param = NULL;
+		      /* skip */
+		    } else if (0 == osip_strcasecmp (u_param->gname, "transport"))
+		      {
+			if (0 == osip_strcasecmp (u_param->gvalue, "udp"))
+			  {
+			    u_param = NULL;
+			    protocol = IPPROTO_UDP;
+			    break;    /* ok */
+			  } else if (0 == osip_strcasecmp (u_param->gvalue, "tcp"))
+			    {
+			      protocol = IPPROTO_TCP;
+			      u_param = NULL;
+			    }
+			break;
+		      }
+		  pos2++;
+		}
+	      
+	      if (u_param == NULL || u_param->gname == NULL || u_param->gvalue == NULL)
+		{
+		  break;            /* default is udp! */
+		}
+	    }
+	  pos++;
+	  co = NULL;
+	}
+      
+      if (co == NULL || co->url == NULL)
+	{
+	  osip_message_free (msg);
+	  OSIP_TRACE (osip_trace
+		      (__FILE__, __LINE__, OSIP_ERROR, NULL,
+		       "eXosip: contact header\n"));
+	  return -1;
+	}
+      
+      /* TODO:
+	 remove extra parameter from new request-uri
+	 check usual parameter like "transport"
+      */
+      
+      if (msg->req_uri!=NULL && msg->req_uri->host!=NULL && co->url->host!=NULL
+	  && 0==osip_strcasecmp(co->url->host, msg->req_uri->host))
+	{
+	  osip_uri_param_t *maddr_param = NULL;
+	  osip_uri_uparam_get_byname (co->url, "maddr", &maddr_param);
+	  if (maddr_param!=NULL && maddr_param->gvalue!=NULL)
+	    {
+	      /* This is a redirect server, the route should probably be removed? */
+	      osip_route_t *route=NULL;
+	      osip_generic_param_t *tag=NULL;
+	      osip_message_get_route (msg, 0, &route);
+	      if (route!=NULL)
+		{
+		  osip_to_get_tag (msg->to, &tag);
+		  if (tag==NULL && route != NULL && route->url != NULL)
+		    {
+		      osip_list_remove(&msg->routes, 0);
+		      osip_route_free(route);
+		    }
+		}
+	    }
+	}
+
+      /* replace request-uri with NEW contact address */
+      osip_uri_free (msg->req_uri);
+      msg->req_uri = NULL;
+      osip_uri_clone (co->url, &msg->req_uri);
+
+	  /* support for diversions headers/draft! */
+	  {
+		  int count=0;
+		  pos=0;
+		  while (!osip_list_eol (&out_tr->last_response->headers, pos))
+		  {
+			  osip_header_t *copy=NULL;
+			  osip_header_t *head = osip_list_get (&out_tr->last_response->headers, pos);
+			  if (head!=NULL && 0==osip_strcasecmp(head->hname, "diversion"))
+			  {
+				  i = osip_header_clone(head, &copy);
+				  if (i==0)
+				  {
+					  osip_list_add (&msg->headers,  copy, count);
+					  count++;
+				  }
+			  }
+			  pos++;
+		  }
+	  }
+
     }
-
-  if (co == NULL || co->url == NULL)
-    {
-      osip_message_free (msg);
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: contact header\n"));
-      return -1;
-    }
-
-  /* TODO:
-     remove extra parameter from new request-uri
-     check usual parameter like "transport"
-   */
-
-  /* replace request-uri with NEW contact address */
-  osip_uri_free (msg->req_uri);
-  msg->req_uri = NULL;
-  osip_uri_clone (co->url, &msg->req_uri);
-
-  /* increment cseq */
-  cseq = atoi (msg->cseq->number);
-  osip_free (msg->cseq->number);
-  msg->cseq->number = strdup_printf ("%i", cseq + 1);
-  if (jd != NULL && jd->d_dialog != NULL)
-    {
-      jd->d_dialog->local_cseq++;
-    }
-
-  i = eXosip_update_top_via(msg);
-  if (i!=0)
-    {
-      osip_message_free (msg);
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: unsupported protocol\n"));
-      return -1;
-    }
-
-  eXosip_add_authentication_information (msg, out_tr->last_response);
-  osip_message_force_update (msg);
-
-  if (0 != osip_strcasecmp (msg->sip_method, "INVITE"))
-    {
-      i = _eXosip_transaction_init (&tr, NICT, eXosip.j_osip, msg);
-  } else
-    {
-      i = _eXosip_transaction_init (&tr, ICT, eXosip.j_osip, msg);
-    }
-
-  if (i != 0)
-    {
-      osip_message_free (msg);
-      return -1;
-    }
-
-  if (out_tr == jc->c_out_tr)
-    {
-      /* replace with the new tr */
-      osip_list_add (eXosip.j_transactions, jc->c_out_tr, 0);
-      jc->c_out_tr = tr;
-
-      /* fix dialog issue */
-      if (jd != NULL)
-        {
-          REMOVE_ELEMENT (jc->c_dialogs, jd);
-          eXosip_dialog_free (jd);
-          jd = NULL;
-        }
-  } else
-    {
-      /* add the new tr for the current dialog */
-      osip_list_add (jd->d_out_trs, tr, 0);
-    }
-
-  sipevent = osip_new_outgoing_sipmessage (msg);
-
-  osip_transaction_set_your_instance (tr, __eXosip_new_jinfo (jc, jd, NULL, NULL));
-  osip_transaction_add_event (tr, sipevent);
-
-  eXosip_update ();             /* fixed? */
-  __eXosip_wakeup ();
-  return 0;
-}
-
-int
-_eXosip_call_send_request_with_credential (eXosip_call_t * jc,
-                                           eXosip_dialog_t * jd,
-                                           osip_transaction_t * out_tr)
-{
-  osip_transaction_t *tr = NULL;
-  osip_message_t *msg = NULL;
-  osip_event_t *sipevent;
-
-  int cseq;
-  osip_via_t *via;
-  int i;
-  int pos;
-
-  if (jc == NULL)
-    return -1;
-  if (jd != NULL)
-    {
-      if (jd->d_out_trs == NULL)
-        return -1;
-    }
-  if (out_tr == NULL
-      || out_tr->orig_request == NULL || out_tr->last_response == NULL)
-    return -1;
-
-  osip_message_clone (out_tr->orig_request, &msg);
-  if (msg == NULL)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: could not clone msg for authentication\n"));
-      return -1;
-    }
-
   /* remove all previous authentication headers */
-  pos = 0;
-  while (!osip_list_eol (&msg->authorizations, pos))
-    {
-      osip_authorization_t *auth;
-
-      auth = (osip_authorization_t *) osip_list_get (&msg->authorizations, pos);
-      osip_list_remove (&msg->authorizations, pos);
-      osip_authorization_free (auth);
-      pos++;
-    }
-
-  pos = 0;
-  while (!osip_list_eol (&msg->proxy_authorizations, pos))
-    {
-      osip_proxy_authorization_t *auth;
-
-      auth =
-        (osip_proxy_authorization_t *) osip_list_get (&msg->
-                                                      proxy_authorizations, pos);
-      osip_list_remove (&msg->proxy_authorizations, pos);
-      osip_authorization_free (auth);
-      pos++;
-    }
-
-
-  via = (osip_via_t *) osip_list_get (&msg->vias, 0);
-  if (via == NULL || msg->cseq == NULL || msg->cseq->number == NULL)
-    {
-      osip_message_free (msg);
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: missing via or cseq header\n"));
-      return -1;
-    }
+  osip_list_special_free(&msg->authorizations, (void *(*)(void *)) &osip_authorization_free);
+  osip_list_special_free(&msg->proxy_authorizations, (void *(*)(void *)) &osip_proxy_authorization_free);
 
   /* increment cseq */
   cseq = atoi (msg->cseq->number);
@@ -1491,7 +1368,11 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
       return -1;
     }
 
-  eXosip_add_authentication_information (msg, out_tr->last_response);
+  if (out_tr->last_response->status_code==401
+      ||out_tr->last_response->status_code==407)
+    eXosip_add_authentication_information (msg, out_tr->last_response);
+  else
+    eXosip_add_authentication_information (msg, NULL);
   osip_message_force_update (msg);
 
   if (0 != osip_strcasecmp (msg->sip_method, "INVITE"))
@@ -1529,7 +1410,11 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
 
   sipevent = osip_new_outgoing_sipmessage (msg);
 
+#ifndef MINISIZE
   osip_transaction_set_your_instance (tr, __eXosip_new_jinfo (jc, jd, NULL, NULL));
+#else
+  osip_transaction_set_your_instance (tr, __eXosip_new_jinfo (jc, jd));
+#endif
   osip_transaction_add_event (tr, sipevent);
 
   eXosip_update ();             /* fixed? */
@@ -1537,6 +1422,8 @@ _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
   return 0;
 }
 
+#ifndef MINISIZE
+
 int
 eXosip_call_get_referto(int did, char *refer_to, size_t refer_to_len)
 {
@@ -1599,3 +1486,63 @@ eXosip_call_get_referto(int did, char *refer_to, size_t refer_to_len)
 
   return 0;
 }
+
+int 
+eXosip_call_find_by_replaces (char* ReplacesStr)
+{
+  eXosip_call_t* jc = NULL;
+  eXosip_dialog_t *jd = NULL;
+  char call_id [256];
+  char* to_tag;
+  char* from_tag;
+  char* early_flag;
+  char* semicolon;
+  
+  /* parse replaces string */
+  strcpy (call_id, ReplacesStr);
+  to_tag = strstr(call_id,"to-tag=");
+  from_tag = strstr(call_id,"from-tag=");
+  early_flag = strstr(call_id,"early-only");
+  
+  if ((to_tag == NULL) ||
+      (from_tag == NULL))
+    return -1;
+  to_tag += strlen ("to-tag=");
+  from_tag += strlen ("from-tag=");
+  
+  while ((semicolon = strrchr(call_id,';')) != NULL)
+    {
+      *semicolon = '\0';
+    }
+  
+  for (jc = eXosip.j_calls; jc != NULL; jc = jc->next)
+    {
+      for (jd = jc->c_dialogs; jd != NULL; jd = jd->next)
+		{
+		  if (jd->d_dialog == NULL)
+			{
+			  /* skip */
+			}    
+		  else if (
+			  ( (0 == strcmp (jd->d_dialog->call_id, call_id)) &&
+			  (0 == strcmp (jd->d_dialog->remote_tag, to_tag)) &&
+			  (0 == strcmp (jd->d_dialog->local_tag, from_tag)) )
+			  ||
+			  ( (0 == strcmp (jd->d_dialog->call_id, call_id)) &&
+			  (0 == strcmp (jd->d_dialog->local_tag, to_tag)) &&
+			  (0 == strcmp (jd->d_dialog->remote_tag, from_tag)) )
+			  )
+			{
+			  /* This dialog match! */
+				if (jd->d_dialog->state == DIALOG_CONFIRMED && early_flag!=NULL)
+					return -3; /* confirmed dialog but already answered with 486 */
+				if (jd->d_dialog->state == DIALOG_EARLY && jd->d_dialog->type == CALLEE)
+					return -2; /* confirmed dialog but already answered with 481 */
+				return jc->c_id; /* match anyway */
+			}
+		}
+	}
+  return -1;   /* answer with 481 */
+}
+
+#endif
