@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,16 +31,21 @@ extern eXosip_t eXosip;
 
 int ipv6_enable = 0;
 
+#ifdef OSIP_MT
 static void *_eXosip_thread (void *arg);
-static int _eXosip_execute (void);
+#endif
 static void _eXosip_keep_alive (void);
 
+#ifndef MINISIZE
+
 void
 eXosip_enable_ipv6 (int _ipv6_enable)
 {
   ipv6_enable = _ipv6_enable;
 }
 
+#endif
+
 void
 eXosip_masquerade_contact (const char *public_address, int port)
 {
@@ -77,27 +82,6 @@ eXosip_masquerade_contact (const char *public_address, int port)
   return;
 }
 
-int
-eXosip_force_masquerade_contact (const char *public_address)
-{
-  if (public_address == NULL || public_address[0] == '\0')
-    {
-      memset (eXosip.net_interfaces[0].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[0].net_firewall_ip));
-      memset (eXosip.net_interfaces[1].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[1].net_firewall_ip));
-      memset (eXosip.net_interfaces[2].net_firewall_ip, '\0',
-              sizeof (eXosip.net_interfaces[2].net_firewall_ip));
-      eXosip.forced_localip = 0;
-      return 0;
-    }
-  eXosip.forced_localip = 1;
-  snprintf (eXosip.net_interfaces[0].net_firewall_ip, 50, "%s", public_address);
-  snprintf (eXosip.net_interfaces[1].net_firewall_ip, 50, "%s", public_address);
-  snprintf (eXosip.net_interfaces[2].net_firewall_ip, 50, "%s", public_address);
-  return 0;
-}
-
 int
 eXosip_guess_localip (int family, char *address, int size)
 {
@@ -163,11 +147,15 @@ eXosip_quit (void)
 {
   jauthinfo_t *jauthinfo;
   eXosip_call_t *jc;
+  eXosip_reg_t *jreg;
+#ifndef MINISIZE
   eXosip_notify_t *jn;
   eXosip_subscribe_t *js;
-  eXosip_reg_t *jreg;
   eXosip_pub_t *jpub;
+#endif
+#ifdef OSIP_MT
   int i;
+#endif
   int pos;
 
   if (eXosip.j_stop_ua==-1)
@@ -182,6 +170,7 @@ eXosip_quit (void)
   __eXosip_wakeup ();
   __eXosip_wakeup_event ();
 
+#ifdef OSIP_MT
   if (eXosip.j_thread!=NULL)
   {
     i = osip_thread_join ((struct osip_thread *) eXosip.j_thread);
@@ -196,6 +185,7 @@ eXosip_quit (void)
 
   jpipe_close (eXosip.j_socketctl);
   jpipe_close (eXosip.j_socketctl_event);
+#endif
 
   osip_free (eXosip.user_agent);
 
@@ -205,6 +195,7 @@ eXosip_quit (void)
       eXosip_call_free (jc);
     }
 
+#ifndef MINISIZE
   for (js = eXosip.j_subscribes; js != NULL; js = eXosip.j_subscribes)
     {
       REMOVE_ELEMENT (eXosip.j_subscribes, js);
@@ -216,10 +207,13 @@ eXosip_quit (void)
       REMOVE_ELEMENT (eXosip.j_notifies, jn);
       eXosip_notify_free (jn);
     }
+#endif
 
+#ifdef OSIP_MT
   osip_mutex_destroy ((struct osip_mutex *) eXosip.j_mutexlock);
 #if !defined (_WIN32_WCE)
   osip_cond_destroy ((struct osip_cond *) eXosip.j_cond);
+#endif
 #endif
 
   if (eXosip.net_interfaces[0].net_socket)
@@ -254,11 +248,13 @@ eXosip_quit (void)
       eXosip_reg_free (jreg);
     }
 
+#ifndef MINISIZE
   for (jpub = eXosip.j_pub; jpub != NULL; jpub = eXosip.j_pub)
     {
       REMOVE_ELEMENT (eXosip.j_pub, jpub);
       _eXosip_pub_free (jpub);
     }
+#endif
 
   while (!osip_list_eol (eXosip.j_transactions, 0))
     {
@@ -322,6 +318,7 @@ eXosip_set_socket (int transport, int socket, int port)
   snprintf (eXosip.net_interfaces[0].net_port,
             sizeof (eXosip.net_interfaces[0].net_port), "%i", port);
 
+#ifdef OSIP_MT
   eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
   if (eXosip.j_thread == NULL)
     {
@@ -330,6 +327,7 @@ eXosip_set_socket (int transport, int socket, int port)
                    "eXosip: Cannot start thread!\n"));
       return -1;
     }
+#endif
   return 0;
 }
 
@@ -369,11 +367,13 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       return -1;
     }
 
+#ifndef MINISIZE
   if (eXosip.http_port)
     {
       /* USE TUNNEL CAPABILITY */
       transport = IPPROTO_TCP;
     }
+#endif
 
   if (port < 0)
     {
@@ -384,6 +384,7 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
     }
 
   net_int->net_ip_family = family;
+#ifndef MINISIZE
   if (family == AF_INET6)
     {
       ipv6_enable = 1;
@@ -391,6 +392,7 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
                   (__FILE__, __LINE__, OSIP_INFO1, NULL,
                    "IPv6 is enabled. Pls report bugs\n"));
     }
+#endif
 
   eXosip_guess_localip (net_int->net_ip_family, localip, sizeof (localip));
   if (localip[0] == '\0')
@@ -431,18 +433,24 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
                            curinfo->ai_protocol);
       if (sock < 0)
         {
-#if !defined(_WIN32_WCE)
+#if defined(_WIN32_WCE)
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: Cannot create socket!\n", strerror (errno)));
+                       "eXosip: Cannot create socket!\n"));
+#else
+          OSIP_TRACE (osip_trace
+                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                       "eXosip: Cannot create socket %s!\n", strerror (errno)));
 #endif
           continue;
         }
 
+#ifndef MINISIZE
       if (eXosip.http_port)
         {
           break;
         }
+#endif
 
       if (curinfo->ai_family == AF_INET6)
         {
@@ -451,10 +459,16 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
             {
               close (sock);
               sock = -1;
+#if defined(_WIN32_WCE)
               OSIP_TRACE (osip_trace
                           (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: Cannot set socket option!\n",
+                           "eXosip: Cannot set socket option!\n"));
+#else
+              OSIP_TRACE (osip_trace
+                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                           "eXosip: Cannot set socket option %s!\n",
                            strerror (errno)));
+#endif
               continue;
             }
 #endif /* IPV6_V6ONLY */
@@ -463,7 +477,12 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       res = bind (sock, curinfo->ai_addr, curinfo->ai_addrlen);
       if (res < 0)
         {
-#if !defined(_WIN32_WCE)
+#if defined(_WIN32_WCE)
+          OSIP_TRACE (osip_trace
+                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                       "eXosip: Cannot bind socket node:%s family:%d\n",
+                       node, curinfo->ai_family));
+#else
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
                        "eXosip: Cannot bind socket node:%s family:%d %s\n",
@@ -477,7 +496,11 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       res = getsockname (sock, (struct sockaddr *) &net_int->ai_addr, &len);
       if (res != 0)
         {
-#if !defined(_WIN32_WCE)
+#if defined(_WIN32_WCE)
+          OSIP_TRACE (osip_trace
+                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                       "eXosip: Cannot get socket name\n"));
+#else
           OSIP_TRACE (osip_trace
                       (__FILE__, __LINE__, OSIP_ERROR, NULL,
                        "eXosip: Cannot get socket name (%s)\n", strerror (errno)));
@@ -490,7 +513,12 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
           res = listen (sock, SOMAXCONN);
           if (res < 0)
             {
-#if !defined(_WIN32_WCE)
+#if defined(_WIN32_WCE)
+              OSIP_TRACE (osip_trace
+                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                           "eXosip: Cannot bind socket node:%s family:%d\n",
+                           node, curinfo->ai_family));
+#else
               OSIP_TRACE (osip_trace
                           (__FILE__, __LINE__, OSIP_ERROR, NULL,
                            "eXosip: Cannot bind socket node:%s family:%d %s\n",
@@ -505,7 +533,7 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       break;
     }
 
-  freeaddrinfo (addrinfo);
+  eXosip_freeaddrinfo (addrinfo);
 
   if (sock < 0)
     {
@@ -515,10 +543,11 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       return -1;
     }
 
-  if (eXosip.http_port)
-    net_int->net_protocol = IPPROTO_UDP;
-  else
+  net_int->net_protocol = IPPROTO_UDP;
+#ifndef MINISIZE
+  if (!eXosip.http_port)
     net_int->net_protocol = transport;
+#endif
   net_int->net_socket = sock;
 
   if (port == 0)
@@ -537,6 +566,7 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
 
 
 
+#ifndef MINISIZE
   if (eXosip.http_port)
     {
       /* only ipv4 */
@@ -581,7 +611,9 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
       } else
         return -1;
     }
+#endif
 
+#ifdef OSIP_MT
   eXosip.j_thread = (void *) osip_thread_create (20000, _eXosip_thread, NULL);
   if (eXosip.j_thread == NULL)
     {
@@ -590,6 +622,7 @@ eXosip_listen_addr (int transport, const char *addr, int port, int family,
                    "eXosip: Cannot start thread!\n"));
       return -1;
     }
+#endif
   return 0;
 }
 
@@ -600,8 +633,10 @@ eXosip_init (void)
 
   memset (&eXosip, 0, sizeof (eXosip));
 
-  eXosip.remove_preloadedroute=1;
   snprintf(eXosip.ipv4_for_gateway, 256, "%s", "217.12.3.11");
+#ifndef MINISIZE
+  snprintf(eXosip.event_package, 256, "%s", "dialog");
+#endif
 
 #ifdef WIN32
   /* Initializing windows socket library */
@@ -626,16 +661,20 @@ eXosip_init (void)
 
   eXosip.j_calls = NULL;
   eXosip.j_stop_ua = 0;
+#ifdef OSIP_MT
   eXosip.j_thread = NULL;
+#endif
   eXosip.j_transactions = (osip_list_t *) osip_malloc (sizeof (osip_list_t));
   osip_list_init (eXosip.j_transactions);
   eXosip.j_reg = NULL;
 
+#ifdef OSIP_MT
 #if !defined (_WIN32_WCE)
   eXosip.j_cond = (struct osip_cond *) osip_cond_init ();
 #endif
 
   eXosip.j_mutexlock = (struct osip_mutex *) osip_mutex_init ();
+#endif
 
   if (-1 == osip_init (&osip))
     {
@@ -651,6 +690,7 @@ eXosip_init (void)
 
   eXosip.j_osip = osip;
 
+#ifdef OSIP_MT
   /* open a TCP socket to wake up the application when needed. */
   eXosip.j_socketctl = jpipe ();
   if (eXosip.j_socketctl == NULL)
@@ -659,6 +699,7 @@ eXosip_init (void)
   eXosip.j_socketctl_event = jpipe ();
   if (eXosip.j_socketctl_event == NULL)
     return -1;
+#endif
 
   /* To be changed in osip! */
   eXosip.j_events = (osip_fifo_t *) osip_malloc (sizeof (osip_fifo_t));
@@ -669,12 +710,13 @@ eXosip_init (void)
 }
 
 
-static int
-_eXosip_execute (void)
+int
+eXosip_execute (void)
 {
   struct timeval lower_tv;
   int i;
 
+#ifdef OSIP_MT
   osip_timers_gettimeout (eXosip.j_osip, &lower_tv);
   if (lower_tv.tv_sec > 15)
     {
@@ -698,6 +740,10 @@ _eXosip_execute (void)
                    "eXosip: timer sec:%i usec:%i!\n",
                    lower_tv.tv_sec, lower_tv.tv_usec));
     }
+#else
+  lower_tv.tv_sec = 0;
+  lower_tv.tv_usec = 0;
+#endif
   i = eXosip_read_message (1, lower_tv.tv_sec, lower_tv.tv_usec);
 
   if (i == -2)
@@ -719,7 +765,9 @@ _eXosip_execute (void)
   /* free all Calls that are in the TERMINATED STATE? */
   eXosip_release_terminated_calls ();
   eXosip_release_terminated_registrations ();
+#ifndef MINISIZE
   eXosip_release_unused_transactions();
+#endif
 
   eXosip_unlock ();
 
@@ -740,6 +788,59 @@ eXosip_set_option (eXosip_option opt, const void *value)
 
   switch (opt)
     {
+      case EXOSIP_OPT_ADD_DNS_CACHE:
+		  {
+			  struct eXosip_dns_cache *entry;
+			  int i;
+			  entry = (struct eXosip_dns_cache*) value;
+			  if (entry==NULL || entry->host[0]=='\0')
+			  {
+				  return -1;
+			  }
+			  for (i=0;i<MAX_EXOSIP_DNS_ENTRY;i++)
+			  {
+				  if (eXosip.dns_entries[i].host[0]!='\0'
+					  &&0==osip_strcasecmp(eXosip.dns_entries[i].host, entry->host))
+				  {
+					  /* update entry */
+					  if (entry->ip[0]!='\0')
+					  {
+						  snprintf(eXosip.dns_entries[i].ip, sizeof(eXosip.dns_entries[i].ip), "%s", entry->ip);
+						  OSIP_TRACE (osip_trace
+										(__FILE__, __LINE__, OSIP_INFO1, NULL,
+										 "eXosip option set: dns cache updated:%s -> %s\n", entry->host, entry->ip));
+					  }
+					  else
+					  {
+						  eXosip.dns_entries[i].host[0]='\0';
+						  OSIP_TRACE (osip_trace
+										(__FILE__, __LINE__, OSIP_INFO2, NULL,
+										 "eXosip option set: dns cache deleted :%s\n", entry->host));
+					  }
+					  return 0;
+				  }
+			  }
+			  if (entry->ip[0]=='\0')
+			  {
+				  return -1;
+			  }
+			  /* not found case: */
+			  for (i=0;i<MAX_EXOSIP_DNS_ENTRY;i++)
+			  {
+				  if (eXosip.dns_entries[i].host[0]=='\0')
+				  {
+					  /* add entry */
+					  snprintf(eXosip.dns_entries[i].host, sizeof(entry->host), "%s", entry->host);
+					  snprintf(eXosip.dns_entries[i].ip, sizeof(entry->ip), "%s", entry->ip);
+					  OSIP_TRACE (osip_trace
+									(__FILE__, __LINE__, OSIP_INFO2, NULL,
+									 "eXosip option set: dns cache added:%s -> %s\n", entry->host, entry->ip));
+					  return 0;
+				  }
+			  }
+			  return -1;
+		  }
+        break;
       case EXOSIP_OPT_UDP_KEEP_ALIVE:
         val = *((int *) value);
         eXosip.keep_alive = val;        /* value in ms */
@@ -748,6 +849,7 @@ eXosip_set_option (eXosip_option opt, const void *value)
         val = *((int *) value);
         eXosip.learn_port = val;        /* 1 to learn port */
         break;
+#ifndef MINISIZE
       case EXOSIP_OPT_SET_HTTP_TUNNEL_PORT:
         val = *((int *) value);
         eXosip.http_port = val; /* value in ms */
@@ -780,6 +882,7 @@ eXosip_set_option (eXosip_option opt, const void *value)
         val = *((int *) value);
         eXosip.dontsend_101 = val;        /* 0 to disable */
         break;
+#endif
 
       case EXOSIP_OPT_USE_RPORT:
         val = *((int *) value);
@@ -797,7 +900,22 @@ eXosip_set_option (eXosip_option opt, const void *value)
                      "eXosip option set: ipv4_for_gateway:%s!\n",
                      eXosip.ipv4_for_gateway));
         break;
+#ifndef MINISIZE
+      case EXOSIP_OPT_EVENT_PACKAGE:
+        tmp = (char *) value;
+        memset (eXosip.event_package, '\0',
+                sizeof (eXosip.event_package));
+        if (tmp != NULL && tmp[0] != '\0')
+          strncpy (eXosip.event_package, tmp, sizeof (eXosip.event_package));
+        OSIP_TRACE (osip_trace
+                    (__FILE__, __LINE__, OSIP_INFO1, NULL,
+                     "eXosip option set: event_package:%s!\n",
+                     eXosip.event_package));
+        break;
+#endif
 
+    default:
+      return -1;
     }
   return 0;
 }
@@ -851,6 +969,7 @@ _eXosip_keep_alive (void)
     }
 }
 
+#ifdef OSIP_MT
 void *
 _eXosip_thread (void *arg)
 {
@@ -858,10 +977,12 @@ _eXosip_thread (void *arg)
 
   while (eXosip.j_stop_ua == 0)
     {
-      i = _eXosip_execute ();
+      i = eXosip_execute ();
       if (i == -2)
         osip_thread_exit ();
     }
   osip_thread_exit ();
   return NULL;
 }
+
+#endif
