@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,9 @@
 
 
 #ifndef  WIN32
+#if !defined(__arc__)
 #include <memory.h>
+#endif
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -41,6 +43,8 @@ static jauthinfo_t *eXosip_find_authentication_info (const char *username,
 
 eXosip_t eXosip;
 
+#ifdef OSIP_MT
+
 void
 __eXosip_wakeup (void)
 {
@@ -53,7 +57,6 @@ __eXosip_wakeup_event (void)
   jpipe_write (eXosip.j_socketctl_event, "w", 1);
 }
 
-
 int
 eXosip_lock (void)
 {
@@ -66,6 +69,8 @@ eXosip_unlock (void)
   return osip_mutex_unlock ((struct osip_mutex *) eXosip.j_mutexlock);
 }
 
+#endif
+
 int
 _eXosip_transaction_init (osip_transaction_t ** transaction,
                        osip_fsm_type_t ctx_type, osip_t * osip,
@@ -112,6 +117,8 @@ eXosip_transaction_find (int tid, osip_transaction_t ** transaction)
   return -1;
 }
 
+#ifndef MINISIZE
+
 static int
 _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
                          int *retry)
@@ -228,7 +235,6 @@ _eXosip_retry_with_auth (eXosip_dialog_t * jd, osip_transaction_t ** ptr,
   return 0;
 }
 
-
 static int
 _eXosip_retry_register_with_auth (eXosip_event_t * je)
 {
@@ -277,7 +283,7 @@ _eXosip_retry_invite_with_auth (eXosip_event_t * je)
   if (*retry < 3)
     {
       (*retry)++;
-      return _eXosip_call_send_request_with_credential(jc, jd, tr);
+      return _eXosip_call_retry_request(jc, jd, tr);
     }
   return -1;
 }
@@ -299,7 +305,7 @@ _eXosip_redirect_invite (eXosip_event_t * je)
       return -1;
     }
   
-  return _eXosip_call_redirect_request (jc, jd, tr);
+  return _eXosip_call_retry_request (jc, jd, tr);
 }
 
 static int
@@ -381,6 +387,7 @@ eXosip_retry_with_auth (eXosip_event_t * je)
     {
       return _eXosip_retry_invite_with_auth (je);
     }
+#ifndef MINISIZE
   else if (je->sid>0)
     {
       return _eXosip_retry_subscribe_with_auth (je);
@@ -391,6 +398,7 @@ eXosip_retry_with_auth (eXosip_event_t * je)
     }
   else if (MSG_IS_PUBLISH (je->request))
     return _eXosip_retry_publish_with_auth (je);
+#endif
   else
     {
       osip_transaction_t *tr=NULL;
@@ -431,7 +439,6 @@ _eXosip_redirect (eXosip_event_t * je)
     }
 }
 
-
 int
 eXosip_default_action (eXosip_event_t * je)
 {
@@ -514,6 +521,7 @@ eXosip_automatic_refresh (void)
         }
     }
 }
+#endif
 
 void
 eXosip_retransmit_lost200ok()
@@ -563,9 +571,11 @@ void
 eXosip_automatic_action (void)
 {
   eXosip_call_t *jc;
-  eXosip_subscribe_t *js;
   eXosip_dialog_t *jd;
+#ifndef MINISIZE
+  eXosip_subscribe_t *js;
   eXosip_notify_t *jn;
+#endif
 
   eXosip_reg_t *jr;
   time_t now;
@@ -600,7 +610,7 @@ eXosip_automatic_action (void)
                 {
                   int i;
 
-                  i = _eXosip_call_send_request_with_credential (jc, NULL, out_tr);
+                  i = _eXosip_call_retry_request (jc, NULL, out_tr);
                   if (i != 0)
                     {
                       OSIP_TRACE (osip_trace
@@ -623,7 +633,7 @@ eXosip_automatic_action (void)
               /* retry with credential */
               int i;
 
-              i = _eXosip_call_redirect_request (jc, NULL, out_tr);
+              i = _eXosip_call_retry_request (jc, NULL, out_tr);
               if (i != 0)
                 {
                   OSIP_TRACE (osip_trace
@@ -662,7 +672,7 @@ eXosip_automatic_action (void)
                       int i;
 
                       i =
-                        _eXosip_call_send_request_with_credential (jc, jd, out_tr);
+                        _eXosip_call_retry_request (jc, jd, out_tr);
                       if (i != 0)
                         {
                           OSIP_TRACE (osip_trace
@@ -685,7 +695,7 @@ eXosip_automatic_action (void)
                   /* retry with credential */
                   int i;
 
-                  i = _eXosip_call_redirect_request (jc, jd, out_tr);
+                  i = _eXosip_call_retry_request (jc, jd, out_tr);
                   if (i != 0)
                     {
                       OSIP_TRACE (osip_trace
@@ -697,6 +707,8 @@ eXosip_automatic_action (void)
         }
     }
 
+#ifndef MINISIZE
+
   for (js = eXosip.j_subscribes; js != NULL; js = js->next)
     {
       if (js->s_id < 1)
@@ -840,6 +852,7 @@ eXosip_automatic_action (void)
         }
     }
 
+#endif
 
   for (jr = eXosip.j_reg; jr != NULL; jr = jr->next)
     {
@@ -884,8 +897,10 @@ eXosip_update ()
 {
   static int static_id = 1;
   eXosip_call_t *jc;
+#ifndef MINISIZE
   eXosip_subscribe_t *js;
   eXosip_notify_t *jn;
+#endif
   eXosip_dialog_t *jd;
   time_t now;
 
@@ -914,6 +929,7 @@ eXosip_update ()
         }
     }
 
+#ifndef MINISIZE
   for (js = eXosip.j_subscribes; js != NULL; js = js->next)
     {
       if (js->s_id < 1)
@@ -955,6 +971,7 @@ eXosip_update ()
             jd->d_id = -1;
         }
     }
+#endif
 }
 
 static jauthinfo_t *
@@ -1076,6 +1093,72 @@ eXosip_add_authentication_information (osip_message_t * req,
       return -1;
    }
 
+  if (last_response==NULL)
+  {
+	  /* we can add all credential that belongs to the same call-id */
+	struct eXosip_http_auth *http_auth;
+	int pos;
+
+	/* update entries with same call_id */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+			continue;
+		if (osip_strcasecmp(http_auth->pszCallId, req->call_id->number)==0)
+		{
+			char *uri;
+
+			authinfo = eXosip_find_authentication_info (req->from->url->username,
+												  http_auth->wa->realm);
+			if (authinfo == NULL)
+			{
+				OSIP_TRACE (osip_trace
+						  (__FILE__, __LINE__, OSIP_INFO2, NULL,
+						   "authinfo: No authentication found for %s %s\n",
+						   req->from->url->username, http_auth->wa->realm));
+				return -1;
+			}
+
+			i = osip_uri_to_str (req->req_uri, &uri);
+			if (i != 0)
+				return -1;
+
+			http_auth->iNonceCount++;
+			if (osip_strcasecmp(req->sip_method, "REGISTER")==0)
+				i = __eXosip_create_authorization_header (http_auth->wa, uri,
+													authinfo->userid,
+													authinfo->passwd,
+													authinfo->ha1, &aut,
+													req->sip_method,
+													http_auth->pszCNonce,
+													http_auth->iNonceCount);
+			else
+				i = __eXosip_create_proxy_authorization_header (http_auth->wa, uri,
+													authinfo->userid,
+													authinfo->passwd,
+													authinfo->ha1, &aut,
+													req->sip_method,
+													http_auth->pszCNonce,
+													http_auth->iNonceCount);
+
+			osip_free (uri);
+			if (i != 0)
+				return -1;
+
+			if (aut != NULL)
+			{
+				if (osip_strcasecmp(req->sip_method, "REGISTER")==0)
+					osip_list_add (&req->authorizations, aut, -1);
+				else
+					osip_list_add (&req->proxy_authorizations, aut, -1);
+				osip_message_force_update (req);
+			}
+		}
+	}
+	return 0;
+  }
+
   pos = 0;
   osip_message_get_www_authenticate (last_response, pos, &wwwauth);
   osip_message_get_proxy_authenticate (last_response, pos, &proxyauth);
@@ -1106,11 +1189,13 @@ eXosip_add_authentication_information (osip_message_t * req,
       if (i != 0)
         return -1;
 
-      i = __eXosip_create_authorization_header (last_response, uri,
+      i = __eXosip_create_authorization_header (wwwauth, uri,
                                                 authinfo->userid,
                                                 authinfo->passwd,
                                                 authinfo->ha1, &aut,
-                                                req->sip_method);
+                                                req->sip_method,
+												"0a4f113b",
+												1);
       osip_free (uri);
       if (i != 0)
         return -1;
@@ -1121,6 +1206,24 @@ eXosip_add_authentication_information (osip_message_t * req,
           osip_message_force_update (req);
         }
 
+	  if (wwwauth->qop_options!=NULL)
+	  {
+		  if (osip_strcasecmp(req->sip_method, "REGISTER")==0
+			  || osip_strcasecmp(req->sip_method, "INVITE")==0
+  			  || osip_strcasecmp(req->sip_method, "SUBSCRIBE")==0)
+		  	  _eXosip_store_nonce(req->call_id->number, wwwauth);
+		  else
+		  {
+			  osip_generic_param_t *to_tag=NULL;
+			  osip_from_param_get_byname (req->to, "tag", &to_tag);
+			  if (to_tag!=NULL)
+			  {
+				  /* if message is part of a dialog */
+			  	  _eXosip_store_nonce(req->call_id->number, wwwauth);
+			  }
+		  }
+	  }
+
       pos++;
       osip_message_get_www_authenticate (last_response, pos, &wwwauth);
     }
@@ -1147,11 +1250,13 @@ eXosip_add_authentication_information (osip_message_t * req,
       if (i != 0)
         return -1;
 
-      i = __eXosip_create_proxy_authorization_header (last_response, uri,
+      i = __eXosip_create_proxy_authorization_header (proxyauth, uri,
                                                       authinfo->userid,
                                                       authinfo->passwd,
                                                       authinfo->ha1,
-                                                      &proxy_aut, req->sip_method);
+                                                      &proxy_aut, req->sip_method,
+													  "0a4f113b",
+													  1);
       osip_free (uri);
       if (i != 0)
         return -1;
@@ -1162,6 +1267,24 @@ eXosip_add_authentication_information (osip_message_t * req,
           osip_message_force_update (req);
         }
 
+	  if (proxyauth->qop_options!=NULL)
+	  {
+		  if (osip_strcasecmp(req->sip_method, "REGISTER")==0
+			  || osip_strcasecmp(req->sip_method, "INVITE")==0
+  			  || osip_strcasecmp(req->sip_method, "SUBSCRIBE")==0)
+		  	  _eXosip_store_nonce(req->call_id->number, proxyauth);
+		  else
+		  {
+			  osip_generic_param_t *to_tag=NULL;
+			  osip_from_param_get_byname (req->to, "tag", &to_tag);
+			  if (to_tag!=NULL)
+			  {
+				  /* if message is part of a dialog */
+			  	  _eXosip_store_nonce(req->call_id->number, proxyauth);
+			  }
+		  }
+	  }
+
       pos++;
       osip_message_get_proxy_authenticate (last_response, pos, &proxyauth);
     }
