@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -25,6 +25,11 @@
 #ifndef __EXOSIP2_H__
 #define __EXOSIP2_H__
 
+#include <stdio.h>
+#ifndef _WIN32_WCE
+#include <errno.h>
+#endif
+
 #ifdef _WIN32_WCE
 #include <stdio.h>
 #include <stdlib.h>
@@ -74,15 +79,57 @@
 
 #endif
 
-#define EXOSIP_VERSION	"3.0.1"
+#define EXOSIP_VERSION	"3.0.2"
 
 #ifdef __cplusplus
 extern "C"
 {
+#endif
+
+#if defined(__arc__)
+#define USE_GETHOSTBYNAME
+#endif
+
+#if defined(USE_GETHOSTBYNAME)
+
+#define NI_MAXHOST      1025
+#define NI_MAXSERV      32
+#define NI_NUMERICHOST  1
+
+#define PF_INET6        AF_INET6
+
+  struct sockaddr_storage {
+    unsigned char sa_len;
+    unsigned char sa_family;         /* Address family AF_XXX */
+    char          sa_data[14];       /* Protocol specific address */
+  };
+  
+  struct addrinfo
+  {
+    int ai_flags;                 /* Input flags.  */
+    int ai_family;                /* Protocol family for socket.  */
+    int ai_socktype;              /* Socket type.  */
+    int ai_protocol;              /* Protocol for socket.  */
+    socklen_t ai_addrlen;         /* Length of socket address.  */
+    struct sockaddr *ai_addr;     /* Socket address for socket.  */
+    char *ai_canonname;           /* Canonical name for service location.  */
+    struct addrinfo *ai_next;     /* Pointer to next in list.  */
+  };
+
+void eXosip_freeaddrinfo(struct addrinfo *ai);
+
+#else
+
+#define eXosip_freeaddrinfo freeaddrinfo 
+
 #endif
 
   void eXosip_update (void);
+#ifdef OSIP_MT
   void __eXosip_wakeup (void);
+#else
+#define __eXosip_wakeup()  ;
+#endif
 
   struct __eXosip_sockaddr
   {
@@ -101,7 +148,7 @@ extern "C"
     osip_dialog_t *d_dialog;    /* active dialog */
 
     time_t d_timer;
-	int d_count;
+    int d_count;
     osip_message_t *d_200Ok;
     osip_message_t *d_ack;
 
@@ -113,6 +160,8 @@ extern "C"
     eXosip_dialog_t *parent;
   };
 
+#ifndef MINISIZE
+
   typedef struct eXosip_subscribe_t eXosip_subscribe_t;
 
   struct eXosip_subscribe_t
@@ -138,7 +187,6 @@ extern "C"
   {
 
     int n_id;
-    char n_uri[255];
     int n_online_status;
 
     int n_ss_status;
@@ -153,6 +201,8 @@ extern "C"
     eXosip_notify_t *parent;
   };
 
+#endif
+
   typedef struct eXosip_call_t eXosip_call_t;
 
   struct eXosip_call_t
@@ -163,8 +213,6 @@ extern "C"
     osip_transaction_t *c_inc_tr;
     osip_transaction_t *c_out_tr;
     int c_retry;                /* avoid too many unsuccessfull retry */
-    osip_transaction_t *c_inc_options_tr;
-    osip_transaction_t *c_out_options_tr;
     void *external_reference;
 
     /* cache old credentials for reuse in next request */
@@ -200,6 +248,8 @@ extern "C"
   };
 
 
+#ifndef MINISIZE
+
   typedef struct eXosip_pub_t eXosip_pub_t;
 
   struct eXosip_pub_t
@@ -223,6 +273,8 @@ extern "C"
   int _eXosip_pub_init (eXosip_pub_t ** pub, const char *aor, const char *exp);
   void _eXosip_pub_free (eXosip_pub_t * pub);
 
+#endif
+
   typedef struct jauthinfo_t jauthinfo_t;
 
   struct jauthinfo_t
@@ -237,26 +289,32 @@ extern "C"
   };
 
   int
-    __eXosip_create_authorization_header (osip_message_t * previous_answer,
+    __eXosip_create_authorization_header (osip_www_authenticate_t *wa,
                                           const char *rquri,
                                           const char *username,
                                           const char *passwd,
                                           const char *ha1,
                                           osip_authorization_t ** auth,
-                                          const char *method);
-  int __eXosip_create_proxy_authorization_header (osip_message_t *
-                                                  previous_answer,
+                                          const char *method,
+										  const char *pszCNonce,
+										  int iNonceCount);
+  int __eXosip_create_proxy_authorization_header (osip_proxy_authenticate_t *wa,
                                                   const char *rquri,
                                                   const char *username,
                                                   const char *passwd,
                                                   const char *ha1,
                                                   osip_proxy_authorization_t
-                                                  ** auth, const char *method);
+                                                  ** auth, const char *method,
+												  const char *pszCNonce,
+												  int iNonceCount);
+  int _eXosip_store_nonce(const char *call_id, osip_proxy_authenticate_t *wa);
+  int _eXosip_delete_nonce(const char *call_id);
 
   eXosip_event_t *eXosip_event_init_for_call (int type, eXosip_call_t * jc,
                                               eXosip_dialog_t * jd,
                                               osip_transaction_t * tr);
 
+#ifndef MINISIZE
   eXosip_event_t *eXosip_event_init_for_subscribe (int type,
                                                    eXosip_subscribe_t * js,
                                                    eXosip_dialog_t * jd,
@@ -265,6 +323,8 @@ extern "C"
                                                 eXosip_notify_t * jn,
                                                 eXosip_dialog_t * jd,
                                                 osip_transaction_t * tr);
+#endif
+
   eXosip_event_t *eXosip_event_init_for_reg (int type, eXosip_reg_t * jr,
                                              osip_transaction_t * tr);
   eXosip_event_t *eXosip_event_init_for_message (int type,
@@ -273,7 +333,6 @@ extern "C"
   int eXosip_event_init (eXosip_event_t ** je, int type);
   void report_call_event (int evt, eXosip_call_t * jc, eXosip_dialog_t * jd,
                           osip_transaction_t * tr);
-  void report_options_event (int evt, osip_transaction_t * tr);
   void report_event (eXosip_event_t * je, osip_message_t * sip);
   int eXosip_event_add (eXosip_event_t * je);
   eXosip_event_t *eXosip_event_wait (int tv_s, int tv_ms);
@@ -320,35 +379,58 @@ extern "C"
   int _eXosip_tcp_connect_socket (char *host, int port);
   int setsockopt_ipv6only (int sock);
 
+#ifndef MINISIZE
+
   int _eXosip_recvfrom (int s, char *buf, int len, unsigned int flags,
                         struct sockaddr *from, socklen_t * fromlen);
   int _eXosip_sendto (int s, const void *buf, size_t len, int flags,
                       const struct sockaddr *to, socklen_t tolen);
 
+#else
+
+#define _eXosip_recvfrom(A, B, C, D, E, F)  recvfrom(A, B, C, D, E, F)
+#define _eXosip_sendto(A, B, C, D, E, F)    sendto(A, B, C, D, E, F)
+
+#endif
+#ifndef MAX_EXOSIP_DNS_ENTRY
+#define MAX_EXOSIP_DNS_ENTRY 10
+#endif
+
+#ifndef MAX_EXOSIP_HTTP_AUTH
+#define MAX_EXOSIP_HTTP_AUTH 10
+#endif
+
   typedef struct eXosip_t eXosip_t;
 
   struct eXosip_t
   {
-    int forced_localip;         /* set to 1 for masquerading contact headers */
     struct eXosip_net net_interfaces[3];
     char *user_agent;
 
     eXosip_call_t *j_calls;     /* my calls        */
+#ifndef MINISIZE
     eXosip_subscribe_t *j_subscribes;   /* my friends      */
     eXosip_notify_t *j_notifies;        /* my susbscribers */
+#endif
     osip_list_t *j_transactions;
 
     eXosip_reg_t *j_reg;        /* my registrations */
+#ifndef MINISIZE
     eXosip_pub_t *j_pub;        /* my publications  */
+#endif
 
+#ifdef OSIP_MT
     void *j_cond;
     void *j_mutexlock;
+#endif
 
     osip_t *j_osip;
     int j_stop_ua;
+#ifdef OSIP_MT
     void *j_thread;
     jpipe_t *j_socketctl;
     jpipe_t *j_socketctl_event;
+#endif
 
     osip_fifo_t *j_events;
 
@@ -356,13 +438,19 @@ extern "C"
 
     int keep_alive;
     int learn_port;
+#ifndef MINISIZE
     int http_port;
     char http_proxy[256];
     char http_outbound_proxy[256];
     int dontsend_101;
+#endif
     int use_rport;
-    int remove_preloadedroute;
     char ipv4_for_gateway[256];
+#ifndef MINISIZE
+    char event_package[256];
+#endif
+    struct eXosip_dns_cache dns_entries[MAX_EXOSIP_DNS_ENTRY];
+    struct eXosip_http_auth http_auths[MAX_EXOSIP_HTTP_AUTH];
   };
 
   typedef struct jinfo_t jinfo_t;
@@ -371,8 +459,10 @@ extern "C"
   {
     eXosip_dialog_t *jd;
     eXosip_call_t *jc;
+#ifndef MINISIZE
     eXosip_subscribe_t *js;
     eXosip_notify_t *jn;
+#endif
   };
 
   int eXosip_guess_ip_for_via (int family, char *address, int size);
@@ -399,8 +489,12 @@ extern "C"
   char *osip_from_tag_new_random (void);
   unsigned int via_branch_new_random (void);
   void __eXosip_delete_jinfo (osip_transaction_t * transaction);
+#ifndef MINISIZE
   jinfo_t *__eXosip_new_jinfo (eXosip_call_t * jc, eXosip_dialog_t * jd,
                                eXosip_subscribe_t * js, eXosip_notify_t * jn);
+#else
+  jinfo_t *__eXosip_new_jinfo (eXosip_call_t * jc, eXosip_dialog_t * jd);
+#endif
 
   int eXosip_dialog_init_as_uac (eXosip_dialog_t ** jd, osip_message_t * _200Ok);
   int eXosip_dialog_init_as_uas (eXosip_dialog_t ** jd,
@@ -441,15 +535,13 @@ extern "C"
   int _eXosip_call_transaction_find (int tid, eXosip_call_t ** jc,
                                      eXosip_dialog_t ** jd,
                                      osip_transaction_t ** tr);
-  int _eXosip_call_redirect_request (eXosip_call_t * jc,
+  int _eXosip_call_retry_request (eXosip_call_t * jc,
                                      eXosip_dialog_t * jd,
                                      osip_transaction_t * out_tr);
-  int _eXosip_call_send_request_with_credential (eXosip_call_t * jc,
-                                                 eXosip_dialog_t * jd,
-                                                 osip_transaction_t * out_tr);
   int eXosip_transaction_find (int tid, osip_transaction_t ** transaction);
   int eXosip_call_dialog_find (int jid, eXosip_call_t ** jc,
                                eXosip_dialog_t ** jd);
+#ifndef MINISIZE
   int _eXosip_insubscription_transaction_find (int tid,
 					       eXosip_notify_t ** jn,
 					       eXosip_dialog_t ** jd,
@@ -460,25 +552,20 @@ extern "C"
 					  eXosip_dialog_t ** jd, osip_transaction_t ** tr);
   int eXosip_subscribe_dialog_find (int nid, eXosip_subscribe_t ** js,
                                     eXosip_dialog_t ** jd);
+#endif
   int eXosip_call_find (int cid, eXosip_call_t ** jc);
   int eXosip_dialog_set_200ok (eXosip_dialog_t * _jd, osip_message_t * _200Ok);
 
-  int _eXosip_answer_invite_3456xx (eXosip_call_t * jc, eXosip_dialog_t * jd,
-                                    int code, osip_message_t ** answer);
-  int _eXosip_answer_invite_2xx (eXosip_call_t * jc, eXosip_dialog_t * jd,
-                                 int code, osip_message_t ** answer);
-  int _eXosip_answer_invite_1xx (eXosip_call_t * jc, eXosip_dialog_t * jd,
-                                 int code, osip_message_t ** answer);
-  int _eXosip_default_answer_invite_1xx (eXosip_call_t * jc,
-                                         eXosip_dialog_t * jd, int code);
-  int _eXosip_default_answer_invite_3456xx (eXosip_call_t * jc,
-                                            eXosip_dialog_t * jd, int code);
+  int _eXosip_answer_invite_123456xx (eXosip_call_t * jc, eXosip_dialog_t * jd,
+				  int code, osip_message_t ** answer, int send);
+#ifndef MINISIZE
   int _eXosip_insubscription_answer_1xx (eXosip_notify_t * jc,
                                          eXosip_dialog_t * jd, int code);
   int _eXosip_insubscription_answer_2xx (eXosip_notify_t * jn,
                                          eXosip_dialog_t * jd, int code);
   int _eXosip_insubscription_answer_3456xx (eXosip_notify_t * jn,
                                             eXosip_dialog_t * jd, int code);
+#endif
 
   int eXosip_build_response_default (int jid, int status);
   int _eXosip_build_response_default (osip_message_t ** dest,
@@ -493,6 +580,7 @@ extern "C"
   void eXosip_kill_transaction (osip_list_t * transactions);
   int eXosip_remove_transaction_from_call (osip_transaction_t * tr,
                                            eXosip_call_t * jc);
+#ifndef MINISIZE
   osip_transaction_t *eXosip_find_last_inc_notify (eXosip_subscribe_t * jn,
                                                    eXosip_dialog_t * jd);
   osip_transaction_t *eXosip_find_last_out_notify (eXosip_notify_t * jn,
@@ -501,6 +589,7 @@ extern "C"
                                                       eXosip_dialog_t * jd);
   osip_transaction_t *eXosip_find_last_out_subscribe (eXosip_subscribe_t * js,
                                                       eXosip_dialog_t * jd);
+#endif
 
   osip_transaction_t *eXosip_find_last_transaction (eXosip_call_t * jc,
                                                     eXosip_dialog_t * jd,
@@ -517,6 +606,9 @@ extern "C"
                                                    eXosip_dialog_t * jd);
   osip_transaction_t *eXosip_find_last_out_invite (eXosip_call_t * jc,
                                                    eXosip_dialog_t * jd);
+  osip_transaction_t *eXosip_find_previous_invite (eXosip_call_t * jc,
+                                                   eXosip_dialog_t * jd,
+                                                   osip_transaction_t *last_invite);
 
 
   int eXosip_call_init (eXosip_call_t ** jc);
@@ -528,6 +620,7 @@ extern "C"
   void eXosip_release_terminated_registrations (void);
   void eXosip_release_unused_transactions(void);
 
+#ifndef MINISIZE
   int eXosip_subscribe_init (eXosip_subscribe_t ** js);
   void eXosip_subscribe_free (eXosip_subscribe_t * js);
   int _eXosip_subscribe_set_refresh_interval (eXosip_subscribe_t * js,
@@ -550,6 +643,7 @@ extern "C"
                                                            jd,
                                                            osip_transaction_t
                                                            * out_tr);
+#endif
 
   int eXosip_is_public_address (const char *addr);
 
