@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -55,6 +55,9 @@ int eXosip_register_remove (int rid)
       return -1;
     }
   jr->r_reg_period = 0;
+  REMOVE_ELEMENT (eXosip.j_reg, jr);
+  eXosip_reg_free (jr);
+  jr=NULL;
   return 0;
 }
 
@@ -103,31 +106,8 @@ _eXosip_register_build_register (eXosip_reg_t * jr, osip_message_t ** _reg)
             int length = strlen (reg->cseq->number);
 
 
-            osip_authorization_t *aut;
-            osip_proxy_authorization_t *proxy_aut;
-
-            aut = (osip_authorization_t *) osip_list_get (&reg->authorizations, 0);
-            while (aut != NULL)
-              {
-                osip_list_remove (&reg->authorizations, 0);
-                osip_authorization_free (aut);
-                aut =
-                  (osip_authorization_t *) osip_list_get (&reg->authorizations, 0);
-              }
-
-            proxy_aut =
-              (osip_proxy_authorization_t *) osip_list_get (&reg->
-                                                            proxy_authorizations,
-                                                            0);
-            while (proxy_aut != NULL)
-              {
-                osip_list_remove (&reg->proxy_authorizations, 0);
-                osip_proxy_authorization_free (proxy_aut);
-                proxy_aut =
-                  (osip_proxy_authorization_t *) osip_list_get (&reg->
-                                                                proxy_authorizations,
-                                                                0);
-              }
+	    osip_list_special_free(&reg->authorizations, (void *(*)(void *)) &osip_authorization_free);
+	    osip_list_special_free(&reg->proxy_authorizations, (void *(*)(void *)) &osip_proxy_authorization_free);
 
             if (0 == osip_strcasecmp (jr->transport, "udp"))
               net = &eXosip.net_interfaces[0];
@@ -187,13 +167,17 @@ _eXosip_register_build_register (eXosip_reg_t * jr, osip_message_t ** _reg)
             reg->cseq->number = (char *) osip_malloc (length + 2);      /* +2 like for 9 to 10 */
             sprintf (reg->cseq->number, "%i", osip_cseq_num);
 
-            {
+			{
               osip_header_t *exp;
 
               osip_message_header_get_byname (reg, "expires", 0, &exp);
-              osip_free (exp->hvalue);
-              exp->hvalue = (char *) osip_malloc (10);
-              snprintf (exp->hvalue, 9, "%i", jr->r_reg_period);
+              if (exp!=NULL)
+			  {
+				  if (exp->hvalue!=NULL)
+					  osip_free (exp->hvalue);
+				  exp->hvalue = (char *) osip_malloc (10);
+				  snprintf (exp->hvalue, 9, "%i", jr->r_reg_period);
+			  }
             }
 
             osip_message_force_update (reg);
@@ -205,6 +189,8 @@ _eXosip_register_build_register (eXosip_reg_t * jr, osip_message_t ** _reg)
                 {
                   eXosip_add_authentication_information (reg, last_response);
                 }
+			  else
+                  eXosip_add_authentication_information (reg, NULL);
               osip_message_free (last_response);
             }
         }
@@ -244,6 +230,9 @@ eXosip_register_build_initial_register (const char *from, const char *proxy,
     {
       if (strcmp (jr->r_aor, from) == 0 && strcmp (jr->r_registrar, proxy) == 0)
         {
+	  REMOVE_ELEMENT (eXosip.j_reg, jr);
+	  eXosip_reg_free (jr);
+	  jr=NULL;
           break;
         }
     }
@@ -271,6 +260,10 @@ eXosip_register_build_initial_register (const char *from, const char *proxy,
 
   /* build register */
   jr->r_reg_period = expires;
+  if (jr->r_reg_period <= 0)      /* too low */
+    jr->r_reg_period = 0;
+  else if (jr->r_reg_period < 100)      /* too low */
+    jr->r_reg_period = 100;
 
   i = _eXosip_register_build_register (jr, reg);
   if (i != 0)
@@ -303,8 +296,8 @@ eXosip_register_build_register (int rid, int expires, osip_message_t ** reg)
     } /* unregistration */
   else if (jr->r_reg_period > 3600)
     jr->r_reg_period = 3600;
-  else if (jr->r_reg_period < 200)      /* too low */
-    jr->r_reg_period = 200;
+  else if (jr->r_reg_period < 100)      /* too low */
+    jr->r_reg_period = 100;
 
   if (jr->r_last_tr != NULL)
     {
