@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002,2003,2004,2005  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <mpatrol.h>
 #endif
 
+#ifndef MINISIZE
 
 #include "eXosip2.h"
 
@@ -196,7 +197,7 @@ eXosip_subscribe_send_initial_request (osip_message_t * subscribe)
   ADD_ELEMENT (eXosip.j_subscribes, js);
   eXosip_update ();             /* fixed? */
   __eXosip_wakeup ();
-  return 0;
+  return js->s_id;
 }
 
 int
@@ -414,3 +415,5 @@ _eXosip_subscribe_send_request_with_credential (eXosip_subscribe_t * js,
   __eXosip_wakeup ();
   return 0;
 }
+
+#endif
