@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -137,40 +137,18 @@ DigestCalcResponse (IN HASHHEX HA1,     /* H(A1) */
   MD5Update (&Md5Ctx, (unsigned char *) ":", 1);
   MD5Update (&Md5Ctx, (unsigned char *) pszDigestUri, strlen (pszDigestUri));
 
-  if (pszQop != NULL)
+  if (pszQop == NULL)
     {
-
-      /*#define AUTH_INT_SUPPORT *//* experimental  */
-#ifdef AUTH_INT_SUPPORT         /* experimental  */
-      char *index = strchr (pszQop, 'i');
-
-      while (index != NULL && index - pszQop >= 5 && strlen (index) >= 3)
-        {
-          if (osip_strncasecmp (index - 5, "auth-int", 8) == 0)
-            {
-              goto auth_withqop;
-            }
-          index = strchr (index + 1, 'i');
-        }
-
-      strchr (pszQop, 'a');
-      while (index != NULL && strlen (index) >= 4)
-        {
-          if (osip_strncasecmp (index - 5, "auth", 4) == 0)
-            {
-              /* and in the case of a unknown token
-                 like auth1. It is not auth, but this
-                 implementation will think it is!??
-                 This is may not happen but it's a bug!
-               */
-              goto auth_withqop;
-            }
-          index = strchr (index + 1, 'a');
-        }
-#endif
       goto auth_withoutqop;
-
-    };
+    }
+  else if (0 == strcmp (pszQop, "auth-int"))
+    {
+      goto auth_withauth_int;
+    }
+  else if (0 == strcmp (pszQop, "auth"))
+    {
+      goto auth_withauth;
+    }
 
 auth_withoutqop:
   MD5Final ((unsigned char *) HA2, &Md5Ctx);
@@ -185,12 +163,12 @@ auth_withoutqop:
 
   goto end;
 
-#ifdef AUTH_INT_SUPPORT         /* experimental  */
-auth_withqop:
-#endif
+auth_withauth_int:
 
   MD5Update (&Md5Ctx, (unsigned char *) ":", 1);
   MD5Update (&Md5Ctx, (unsigned char *) HEntity, HASHHEXLEN);
+
+auth_withauth:
   MD5Final ((unsigned char *) HA2, &Md5Ctx);
   CvtHex (HA2, HA2Hex);
 
@@ -215,16 +193,17 @@ end:
 
 
 int
-__eXosip_create_authorization_header (osip_message_t * previous_answer,
+__eXosip_create_authorization_header (osip_www_authenticate_t *wa,
                                       const char *rquri, const char *username,
                                       const char *passwd, const char *ha1,
                                       osip_authorization_t ** auth,
-                                      const char *method)
+                                      const char *method,
+									  const char *pCNonce,
+									  int iNonceCount)
 {
   osip_authorization_t *aut;
-  osip_www_authenticate_t *wa = NULL;
 
-  osip_message_get_www_authenticate (previous_answer, 0, &wa);
+  char *qop=NULL;
 
   /* make some test */
   if (passwd == NULL)
@@ -286,6 +265,11 @@ __eXosip_create_authorization_header (osip_message_t * previous_answer,
 
   osip_authorization_set_algorithm (aut, osip_strdup ("MD5"));
 
+  qop = osip_www_authenticate_get_qop_options (wa);
+  if (qop==NULL || qop[0]=='\0' || strlen(qop)<4)
+    qop=NULL;
+
+
   {
     char *pszNonce =
       osip_strdup_without_quote (osip_www_authenticate_get_nonce (wa));
@@ -305,17 +289,34 @@ __eXosip_create_authorization_header (osip_message_t * previous_answer,
     HASHHEX Response;
     const char *pha1 = NULL;
 
+	if (qop!=NULL)
+      {
+	    /* only accept qop="auth" */
+	    pszQop = osip_strdup("auth");
+
+		szNonceCount = osip_malloc(10);
+		snprintf(szNonceCount, 9, "%.8i", iNonceCount);
+
+		pszCNonce = osip_strdup (pCNonce);
+	
+		osip_authorization_set_message_qop (aut, osip_strdup ("auth"));
+		osip_authorization_set_nonce_count (aut, osip_strdup (szNonceCount));
+	
+		{
+		  char *tmp = osip_malloc (strlen (pszCNonce) + 3);
+		  sprintf (tmp, "\"%s\"", pszCNonce);
+		  osip_authorization_set_cnonce (aut, tmp);
+		}
+      }
+
     pszPass = passwd;
-    if (osip_authorization_get_nonce_count (aut) != NULL)
-      szNonceCount = osip_strdup (osip_authorization_get_nonce_count (aut));
-    if (osip_authorization_get_message_qop (aut) != NULL)
-      pszQop = osip_strdup (osip_authorization_get_message_qop (aut));
 
     if (ha1 && ha1[0])
       {
         /* Depending on algorithm=md5 */
         pha1 = ha1;
-    } else
+      }
+    else
       {
         DigestCalcHA1 (pszAlg, pszUser, pszRealm, pszPass, pszNonce,
                        pszCNonce, HA1);
@@ -346,18 +347,19 @@ __eXosip_create_authorization_header (osip_message_t * previous_answer,
 }
 
 int
-__eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
-                                            const char *rquri,
+__eXosip_create_proxy_authorization_header (osip_proxy_authenticate_t *wa,
+											const char *rquri,
                                             const char *username,
                                             const char *passwd,
                                             const char *ha1,
                                             osip_proxy_authorization_t **
-                                            auth, const char *method)
+                                            auth, const char *method,
+											const char *pCNonce,
+											int iNonceCount)
 {
   osip_proxy_authorization_t *aut;
-  osip_proxy_authenticate_t *wa;
 
-  osip_message_get_proxy_authenticate (previous_answer, 0, &wa);
+  char *qop=NULL;
 
   /* make some test */
   if (passwd == NULL)
@@ -419,6 +421,10 @@ __eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
   }
   osip_proxy_authorization_set_algorithm (aut, osip_strdup ("MD5"));
 
+  qop = osip_www_authenticate_get_qop_options (wa);
+  if (qop==NULL || qop[0]=='\0' || strlen(qop)<4)
+    qop=NULL;
+
   {
     char *pszNonce = NULL;
     char *pszCNonce = NULL;
@@ -443,18 +449,26 @@ __eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
       return -1;
     pszNonce = osip_strdup_without_quote (osip_www_authenticate_get_nonce (wa));
 
-    /* should upgrade szNonceCount */
-    /* should add szNonceCount in aut */
-    /* should upgrade pszCNonce */
-    /* should add pszCNonce in aut */
-
-    if (osip_proxy_authenticate_get_qop_options (wa) != NULL)
+    if (qop!=NULL)
       {
-        szNonceCount = osip_strdup ("00000001");
-        /* MUST be incremented on each */
-        pszQop = osip_strdup (osip_proxy_authenticate_get_qop_options (wa));
-        pszCNonce = osip_strdup ("234abcc436e2667097e7fe6eia53e8dd");
+	    /* only accept qop="auth" */
+	    pszQop = osip_strdup("auth");
+		
+		szNonceCount = osip_malloc(10);
+		snprintf(szNonceCount, 9, "%.8i", iNonceCount);
+
+		pszCNonce = osip_strdup (pCNonce);
+
+		osip_proxy_authorization_set_message_qop (aut, osip_strdup ("auth"));
+		osip_proxy_authorization_set_nonce_count (aut, osip_strdup (szNonceCount));
+
+		{
+		  char *tmp = osip_malloc (strlen (pszCNonce) + 3);
+		  sprintf (tmp, "\"%s\"", pszCNonce);
+		  osip_proxy_authorization_set_cnonce (aut, tmp);
+		}
       }
+
     if (ha1 && ha1[0])
       {
         /* Depending on algorithm=md5 */
@@ -487,3 +501,72 @@ __eXosip_create_proxy_authorization_header (osip_message_t * previous_answer,
   *auth = aut;
   return 0;
 }
+
+int _eXosip_store_nonce(const char *call_id, osip_proxy_authenticate_t *wa)
+{
+	struct eXosip_http_auth *http_auth;
+	int pos;
+
+	/* update entries with same call_id */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+			continue;
+		if (osip_strcasecmp(http_auth->pszCallId, call_id)==0
+			&& osip_strcasecmp(http_auth->wa->realm, wa->realm)==0)
+		{
+			osip_proxy_authenticate_free(http_auth->wa);
+			http_auth->wa=NULL;
+			osip_proxy_authenticate_clone(wa, &(http_auth->wa));
+			http_auth->iNonceCount = 1;
+			if (http_auth->wa==NULL)
+				memset(http_auth, 0, sizeof(struct eXosip_http_auth));
+			return 0;
+		}
+	}
+
+	/* not found */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+		{
+			snprintf(http_auth->pszCallId, sizeof(http_auth->pszCallId),
+				call_id);
+			snprintf(http_auth->pszCNonce, sizeof(http_auth->pszCNonce),
+				"0a4f113b");
+			http_auth->iNonceCount = 1;
+			osip_proxy_authenticate_clone(wa, &(http_auth->wa));
+			if (http_auth->wa==NULL)
+				memset(http_auth, 0, sizeof(struct eXosip_http_auth));
+			return 0;
+		}
+	}
+
+	OSIP_TRACE (osip_trace
+                (__FILE__, __LINE__, OSIP_ERROR, NULL,
+                 "Compile with higher MAX_EXOSIP_HTTP_AUTH value (current=%i)\n", MAX_EXOSIP_HTTP_AUTH));
+	return -1;
+}
+
+int _eXosip_delete_nonce(const char *call_id)
+{
+	struct eXosip_http_auth *http_auth;
+	int pos;
+
+	/* update entries with same call_id */
+	for (pos=0;pos<MAX_EXOSIP_HTTP_AUTH;pos++)
+	{
+		http_auth = &eXosip.http_auths[pos];
+		if (http_auth->pszCallId[0]=='\0')
+			continue;
+		if (osip_strcasecmp(http_auth->pszCallId, call_id)==0)
+		{
+			osip_proxy_authenticate_free(http_auth->wa);
+			memset(http_auth, 0, sizeof(struct eXosip_http_auth));
+			return 0;
+		}
+	}
+	return -1;
+}
