@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -150,7 +150,7 @@ _eXosip_dialog_add_contact(osip_message_t *request, osip_message_t *answer)
         if (i == 0)
         {
             memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-            freeaddrinfo (addrinfo);
+            eXosip_freeaddrinfo (addrinfo);
             c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
             OSIP_TRACE (osip_trace (__FILE__, __LINE__, OSIP_INFO1, NULL,
                         "eXosip: here is the resolved destination host=%s\n",
@@ -165,11 +165,7 @@ _eXosip_dialog_add_contact(osip_message_t *request, osip_message_t *answer)
 
     if (locip[0] == '\0')
 	{
-#ifndef SM
 		eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
-#else
-		eXosip_get_localip_for (request->req_uri->host, locip, 49);
-#endif
 		if (locip[0]=='\0')
 		{
 			OSIP_TRACE (osip_trace
@@ -198,7 +194,7 @@ _eXosip_dialog_add_contact(osip_message_t *request, osip_message_t *answer)
 	if (osip_strcasecmp(transport, "UDP")!=0)
 	{
 		contact[strlen(contact)-1]='\0';
-		strcat(contact, "transport=");
+		strcat(contact, ";transport=");
 		strcat(contact, transport);
 		strcat(contact, ">");
 	}
@@ -304,7 +300,6 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
 
   /*guess the local ip since req uri is known */
   memset(locip, '\0', sizeof(locip));
-#ifndef SM
   eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
   if (locip[0]=='\0')
     {
@@ -313,7 +308,6 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
                    "eXosip: no default interface defined\n"));
       return -1;
     }
-#endif
 
   i = osip_message_init (&request);
   if (i != 0)
@@ -348,6 +342,44 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
                        to));
           goto brood_error_1;
         }
+
+	  /* REMOVE ALL URL PARAMETERS from to->url headers and add them as headers */
+	  if (request->to!=NULL &&
+		  request->to->url!=NULL)
+	  {
+		  osip_uri_t *url = request->to->url;
+		  while (osip_list_size(&url->url_headers)>0)
+		  {
+			  osip_uri_header_t *u_header;
+			  u_header = (osip_uri_param_t *) osip_list_get (&url->url_headers, 0);
+			  if (u_header==NULL)
+				break;
+
+			  if (osip_strcasecmp(u_header->gname, "from")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "to")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "call-id")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "cseq")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "via")==0)
+			  {
+			  }
+			  else if (osip_strcasecmp(u_header->gname, "contact")==0)
+			  {
+			  }
+			  else
+				  osip_message_set_header(request, u_header->gname, u_header->gvalue);
+			  osip_list_remove (&url->url_headers, 0);
+			  osip_uri_param_free (u_header);
+		  }
+	  }
+
       if (proxy != NULL && proxy[0] != 0)
         {                       /* equal to a pre-existing route set */
           /* if the pre-existing route set contains a "lr" (compliance
@@ -395,9 +427,6 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
         }
     }
 
-#ifdef SM
-  eXosip_get_localip_for (request->req_uri->host, locip, 49);
-#endif
   if (locip[0]=='\0')
     goto brood_error_1;
 
@@ -444,27 +473,6 @@ generating_request_out_of_dialog (osip_message_t ** dest, const char *method,
   /* always add the Max-Forward header */
   osip_message_set_max_forwards (request, "70"); /* a UA should start a request with 70 */
 
-  /* add specific headers for each kind of request... */
-
-  if (0 == strcmp ("INVITE", method) || 0 == strcmp ("SUBSCRIBE", method))
-    {
-
-      /* This is probably useless for other messages */
-#if 0
-      osip_message_set_allow (request, "INVITE");
-      osip_message_set_allow (request, "ACK");
-      osip_message_set_allow (request, "UPDATE");
-      osip_message_set_allow (request, "INFO");
-      osip_message_set_allow (request, "CANCEL");
-      osip_message_set_allow (request, "BYE");
-      osip_message_set_allow (request, "OPTIONS");
-      osip_message_set_allow (request, "REFER");
-      osip_message_set_allow (request, "SUBSCRIBE");
-      osip_message_set_allow (request, "NOTIFY");
-      osip_message_set_allow (request, "MESSAGE");
-#endif
-    }
-
   if (0 == strcmp ("REGISTER", method))
     {
   } else if (0 == strcmp ("INFO", method))
@@ -517,11 +525,7 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
     }
 
   memset(locip, '\0', sizeof(locip));
-#ifdef SM
-  eXosip_get_localip_for ((*reg)->req_uri->host, locip, 49);
-#else
   eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
-#endif
 
   if (locip[0]=='\0')
     {
@@ -542,7 +546,12 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
       if (i == 0 && a_from != NULL
           && a_from->url != NULL && a_from->url->username != NULL)
         {
-          contact = (char *) osip_malloc (50 + strlen (a_from->url->username));
+	  int len = 50 + strlen (a_from->url->username);
+	  if (transport!=NULL && osip_strcasecmp(transport, "UDP")!=0)
+	    {
+	      len = len + strlen(transport) + 12; /* strlen(";transport=") */
+	    }
+	  contact = (char *) osip_malloc (len);
           if (eXosip.net_interfaces[0].net_firewall_ip[0] != '\0')
             {
               char *c_address = (*reg)->req_uri->host;
@@ -556,7 +565,7 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
               if (i == 0)
                 {
                   memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-                  freeaddrinfo (addrinfo);
+                  eXosip_freeaddrinfo (addrinfo);
                   c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
                   OSIP_TRACE (osip_trace
                               (__FILE__, __LINE__, OSIP_INFO1, NULL,
@@ -579,6 +588,14 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
               sprintf (contact, "<sip:%s@%s:%s>", a_from->url->username,
                        locip, net->net_port);
             }
+	  
+	  if (transport!=NULL && osip_strcasecmp(transport, "UDP")!=0)
+	    {
+	      contact[strlen(contact)-1]='\0';
+	      strcat(contact, ";transport=");
+	      strcat(contact, transport);
+	      strcat(contact, ">");
+	    }
 
           osip_message_set_contact (*reg, contact);
           osip_free (contact);
@@ -601,6 +618,8 @@ generating_register (osip_message_t ** reg, char *transport, char *from,
   return 0;
 }
 
+#ifndef MINISIZE
+
 int
 generating_publish (osip_message_t ** message, const char *to,
                     const char *from, const char *route)
@@ -623,6 +642,8 @@ generating_publish (osip_message_t ** message, const char *to,
   return 0;
 }
 
+#endif
+
 static int
 dialog_fill_route_set (osip_dialog_t * dialog, osip_message_t * request)
 {
@@ -744,11 +765,7 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
     }
 
   memset(locip, '\0', sizeof(locip));
-#ifdef SM
-  eXosip_get_localip_for (dialog->remote_contact_uri->url->host, locip, 49);
-#else
   eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
-#endif
   if (locip[0]=='\0')
     {
       OSIP_TRACE (osip_trace
@@ -843,7 +860,7 @@ _eXosip_build_request_within_dialog (osip_message_t ** dest,
         if (i == 0)
           {
             memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-            freeaddrinfo (addrinfo);
+            eXosip_freeaddrinfo (addrinfo);
             c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
             OSIP_TRACE (osip_trace
                         (__FILE__, __LINE__, OSIP_INFO1, NULL,
