@@ -1,6 +1,6 @@
 /*
   eXosip - This is the eXtended osip library.
-  Copyright (C) 2002, 2003  Aymeric MOIZARD  - jack@atosc.org
+  Copyright (C) 2002,2003,2004,2005,2006,2007  Aymeric MOIZARD  - jack@atosc.org
   
   eXosip is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@ _eXosip_build_response_default (osip_message_t ** dest,
   sprintf (response->sip_version, "SIP/2.0");
   osip_message_set_status_code (response, status);
 
+#ifndef MINISIZE
   /* handle some internal reason definitions. */
   if (MSG_IS_NOTIFY (request) && status == 481)
     {
@@ -71,6 +72,18 @@ _eXosip_build_response_default (osip_message_t ** dest,
       response->req_uri = NULL;
       response->sip_method = NULL;
     }
+#else
+  response->reason_phrase = osip_strdup (osip_message_get_reason (status));
+  if (response->reason_phrase == NULL)
+    {
+      if (response->status_code == 101)
+	response->reason_phrase = osip_strdup ("Dialog Establishement");
+      else
+	response->reason_phrase = osip_strdup ("Unknown code");
+    }
+  response->req_uri = NULL;
+  response->sip_method = NULL;
+#endif
 
   i = osip_to_clone (request->to, &(response->to));
   if (i != 0)
@@ -115,6 +128,7 @@ _eXosip_build_response_default (osip_message_t ** dest,
   if (i != 0)
     goto grd_error_1;
 
+#ifndef MINISIZE
   if (MSG_IS_SUBSCRIBE (request))
     {
       osip_header_t *exp;
@@ -135,19 +149,6 @@ _eXosip_build_response_default (osip_message_t ** dest,
             osip_list_add (&response->headers, cp, 0);
         }
     }
-
-#if 0
-  osip_message_set_allow (response, "INVITE");
-  osip_message_set_allow (response, "ACK");
-  osip_message_set_allow (response, "OPTIONS");
-  osip_message_set_allow (response, "CANCEL");
-  osip_message_set_allow (response, "BYE");
-  osip_message_set_allow (response, "SUBSCRIBE");
-  osip_message_set_allow (response, "NOTIFY");
-  osip_message_set_allow (response, "MESSAGE");
-  osip_message_set_allow (response, "INFO");
-  osip_message_set_allow (response, "REFER");
-  osip_message_set_allow (response, "UPDATE");
 #endif
 
   osip_message_set_user_agent (response, eXosip.user_agent);
@@ -204,11 +205,7 @@ complete_answer_that_establish_a_dialog (osip_message_t * response,
     }
 
   memset(locip, '\0', sizeof(locip));
-#ifdef SM
-  eXosip_get_localip_from_via (response, locip, 49);
-#else
   eXosip_guess_ip_for_via (net->net_ip_family, locip, 49);
-#endif
 
   if (request->to->url->username == NULL)
     snprintf (contact, 1000, "<sip:%s:%s>", locip, net->net_port);
@@ -231,7 +228,7 @@ complete_answer_that_establish_a_dialog (osip_message_t * response,
           if (i == 0)
             {
               memcpy (&addr, addrinfo->ai_addr, addrinfo->ai_addrlen);
-              freeaddrinfo (addrinfo);
+              eXosip_freeaddrinfo (addrinfo);
               c_address = inet_ntoa (((struct sockaddr_in *) &addr)->sin_addr);
               OSIP_TRACE (osip_trace
                           (__FILE__, __LINE__, OSIP_INFO1, NULL,
@@ -254,66 +251,30 @@ complete_answer_that_establish_a_dialog (osip_message_t * response,
         }
     }
 
+    {
+      osip_via_t *via;
+      
+      via = (osip_via_t *) osip_list_get (&response->vias, 0);
+      if (via == NULL || via->protocol == NULL)
+	return -1;
+      if (strlen(contact)+strlen(via->protocol)<1000
+	  && 0 != osip_strcasecmp (via->protocol, "UDP"))
+	{
+	  contact[strlen(contact)-1]='\0';
+	  strcat(contact, ";transport=");
+	  strcat(contact, via->protocol);
+	  strcat(contact, ">");
+	}
+    }
+
   osip_message_set_contact (response, contact);
 
   return 0;
 }
 
 int
-_eXosip_answer_invite_1xx (eXosip_call_t * jc, eXosip_dialog_t * jd, int code,
-                           osip_message_t ** answer)
-{
-  int i;
-  osip_transaction_t *tr;
-
-  *answer = NULL;
-  tr = eXosip_find_last_inc_invite (jc, jd);
-  if (tr == NULL)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: cannot find transaction to answer"));
-      return -1;
-    }
-  /* is the transaction already answered? */
-  if (tr->state == IST_COMPLETED
-      || tr->state == IST_CONFIRMED || tr->state == IST_TERMINATED)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: transaction already answered\n"));
-      return -1;
-    }
-
-  if (jd == NULL)
-    i = _eXosip_build_response_default (answer, NULL, code, tr->orig_request);
-  else
-    i =
-      _eXosip_build_response_default (answer, jd->d_dialog, code,
-                                      tr->orig_request);
-
-  if (i != 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "ERROR: Could not create response for invite\n"));
-      return -2;
-    }
-
-  osip_message_set_content_length (*answer, "0");
-  /*  send message to transaction layer */
-
-  if (code > 100)
-    {
-      i = complete_answer_that_establish_a_dialog (*answer, tr->orig_request);
-    }
-
-  return 0;
-}
-
-int
-_eXosip_answer_invite_2xx (eXosip_call_t * jc, eXosip_dialog_t * jd, int code,
-                           osip_message_t ** answer)
+_eXosip_answer_invite_123456xx (eXosip_call_t * jc, eXosip_dialog_t * jd, int code,
+                           osip_message_t ** answer, int send)
 {
   int i;
   osip_transaction_t *tr;
@@ -329,7 +290,7 @@ _eXosip_answer_invite_2xx (eXosip_call_t * jc, eXosip_dialog_t * jd, int code,
       return -1;
     }
 
-  if (jd != NULL && jd->d_dialog == NULL)
+  if (code>=200 && code<300 && jd != NULL && jd->d_dialog == NULL)
     {                           /* element previously removed */
       OSIP_TRACE (osip_trace
                   (__FILE__, __LINE__, OSIP_ERROR, NULL,
@@ -364,191 +325,41 @@ _eXosip_answer_invite_2xx (eXosip_call_t * jc, eXosip_dialog_t * jd, int code,
 
   /* request that estabish a dialog: */
   /* 12.1.1 UAS Behavior */
+  if (code > 100 && code <300)
   {
     i = complete_answer_that_establish_a_dialog (*answer, tr->orig_request);
     if (i != 0)
       goto g2atii_error_1;;     /* ?? */
   }
 
+
+  if (send==1)
+    {
+      osip_event_t *evt_answer;
+      if (code >= 200 && code <300 && jd!=NULL)
+	{
+	  eXosip_dialog_set_200ok (jd, *answer);
+	  /* wait for a ACK */
+	  osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
+	}
+
+      evt_answer = osip_new_outgoing_sipmessage (*answer);
+      evt_answer->transactionid = tr->transactionid;
+      
+      osip_transaction_add_event (tr, evt_answer);
+      __eXosip_wakeup ();
+      *answer=NULL;
+    }
+
   return 0;
 
 g2atii_error_1:
   osip_message_free (*answer);
+  *answer=NULL;
   return -1;
 }
 
-int
-_eXosip_answer_invite_3456xx (eXosip_call_t * jc, eXosip_dialog_t * jd,
-                              int code, osip_message_t ** answer)
-{
-  int i;
-  osip_transaction_t *tr;
-
-  *answer = NULL;
-  tr = eXosip_find_last_inc_invite (jc, jd);
-  if (tr == NULL)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: cannot find transaction to answer"));
-      return -1;
-    }
-  /* is the transaction already answered? */
-  if (tr->state == IST_COMPLETED
-      || tr->state == IST_CONFIRMED || tr->state == IST_TERMINATED)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: transaction already answered\n"));
-      return -1;
-    }
-
-  i =
-    _eXosip_build_response_default (answer, jd->d_dialog, code, tr->orig_request);
-  if (i != 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "ERROR: Could not create response for invite\n"));
-      return -1;
-    }
-
-  if ((300 <= code) && (code <= 399))
-    {
-      /* Should add contact fields */
-      /* ... */
-    }
-
-  osip_message_set_content_length (*answer, "0");
-  /*  send message to transaction layer */
-
-  return 0;
-}
-
-int
-_eXosip_default_answer_invite_1xx (eXosip_call_t * jc, eXosip_dialog_t * jd,
-                                   int code)
-{
-  osip_event_t *evt_answer;
-  osip_message_t *response;
-  int i;
-  osip_transaction_t *tr;
-
-  tr = eXosip_find_last_inc_invite (jc, jd);
-  if (tr == NULL)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: cannot find transaction to answer"));
-      return -1;
-    }
-  /* is the transaction already answered? */
-  if (tr->state == IST_COMPLETED
-      || tr->state == IST_CONFIRMED || tr->state == IST_TERMINATED)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: transaction already answered\n"));
-      return -1;
-    }
-
-  if (jd == NULL)
-    i = _eXosip_build_response_default (&response, NULL, code, tr->orig_request);
-  else
-    i =
-      _eXosip_build_response_default (&response, jd->d_dialog, code,
-                                      tr->orig_request);
-
-  if (i != 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "ERROR: Could not create response for invite\n"));
-      return -2;
-    }
-
-  osip_message_set_content_length (response, "0");
-  /*  send message to transaction layer */
-
-  if (code > 100)
-    {
-      /* request that estabish a dialog: */
-      /* 12.1.1 UAS Behavior */
-      i = complete_answer_that_establish_a_dialog (response, tr->orig_request);
-
-      if (jd == NULL)
-        {
-          i = eXosip_dialog_init_as_uas (&jd, tr->orig_request, response);
-          if (i != 0)
-            {
-              OSIP_TRACE (osip_trace
-                          (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                           "eXosip: cannot create dialog!\n"));
-          } else
-            {
-              ADD_ELEMENT (jc->c_dialogs, jd);
-            }
-        }
-    }
-
-  evt_answer = osip_new_outgoing_sipmessage (response);
-  evt_answer->transactionid = tr->transactionid;
-
-  osip_transaction_add_event (tr, evt_answer);
-  __eXosip_wakeup ();
-
-  return 0;
-}
-
-int
-_eXosip_default_answer_invite_3456xx (eXosip_call_t * jc,
-                                      eXosip_dialog_t * jd, int code)
-{
-  osip_event_t *evt_answer;
-  osip_message_t *response;
-  int i;
-  osip_transaction_t *tr;
-
-  tr = eXosip_find_last_inc_invite (jc, jd);
-  if (tr == NULL)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: cannot find transaction to answer"));
-      return -1;
-    }
-  /* is the transaction already answered? */
-  if (tr->state == IST_COMPLETED
-      || tr->state == IST_CONFIRMED || tr->state == IST_TERMINATED)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: transaction already answered\n"));
-      return -1;
-    }
-
-  i =
-    _eXosip_build_response_default (&response, jd->d_dialog, code,
-                                    tr->orig_request);
-  if (i != 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "ERROR: Could not create response for invite\n"));
-      return -1;
-    }
-
-  osip_message_set_content_length (response, "0");
-  /*  send message to transaction layer */
-
-  evt_answer = osip_new_outgoing_sipmessage (response);
-  evt_answer->transactionid = tr->transactionid;
-
-  osip_transaction_add_event (tr, evt_answer);
-  __eXosip_wakeup ();
-  return 0;
-}
-
+#ifndef MINISIZE
 
 int
 _eXosip_insubscription_answer_1xx (eXosip_notify_t * jn, eXosip_dialog_t * jd,
@@ -610,91 +421,6 @@ _eXosip_insubscription_answer_1xx (eXosip_notify_t * jn, eXosip_dialog_t * jd,
   return 0;
 }
 
-#if 0
-int
-_eXosip_insubscription_answer_2xx (eXosip_notify_t * jn, eXosip_dialog_t * jd,
-                                   int code)
-{
-  osip_event_t *evt_answer;
-  osip_message_t *response;
-  int i;
-  osip_transaction_t *tr;
-
-  tr = eXosip_find_last_inc_subscribe (jn, jd);
-
-  if (tr == NULL || tr->orig_request == NULL)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: cannot find transaction to answer\n"));
-      return -1;
-    }
-
-  if (jd != NULL && jd->d_dialog == NULL)
-    {                           /* element previously removed, this is a no hop! */
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                   "eXosip: cannot answer this closed transaction\n"));
-      return -1;
-    }
-
-  if (jd == NULL)
-    i = _eXosip_build_response_default (&response, NULL, code, tr->orig_request);
-  else
-    i =
-      _eXosip_build_response_default (&response, jd->d_dialog, code,
-                                      tr->orig_request);
-
-  if (i != 0)
-    {
-      OSIP_TRACE (osip_trace
-                  (__FILE__, __LINE__, OSIP_INFO1, NULL,
-                   "ERROR: Could not create response for subscribe\n"));
-      code = 500;               /* ? which code to use? */
-      return -1;
-    }
-
-  /* request that estabish a dialog: */
-  /* 12.1.1 UAS Behavior */
-  {
-    i = complete_answer_that_establish_a_dialog (response, tr->orig_request);
-    if (i != 0)
-      goto g2atii_error_1;;     /* ?? */
-  }
-
-  /* THIS RESPONSE MUST BE SENT RELIABILY until the final ACK is received !! */
-  /* this response must be stored at the upper layer!!! (it will be destroyed */
-  /* right after being sent! */
-
-  if (jd == NULL)
-    {
-      i = eXosip_dialog_init_as_uas (&jd, tr->orig_request, response);
-      if (i != 0)
-        {
-          OSIP_TRACE (osip_trace
-                      (__FILE__, __LINE__, OSIP_ERROR, NULL,
-                       "eXosip: cannot create dialog!\n"));
-          return -1;
-        }
-      ADD_ELEMENT (jn->n_dialogs, jd);
-    }
-
-  eXosip_dialog_set_200ok (jd, response);
-  evt_answer = osip_new_outgoing_sipmessage (response);
-  evt_answer->transactionid = tr->transactionid;
-
-  osip_transaction_add_event (tr, evt_answer);
-  __eXosip_wakeup ();
-
-  osip_dialog_set_state (jd->d_dialog, DIALOG_CONFIRMED);
-  return 0;
-
-g2atii_error_1:
-  osip_message_free (response);
-  return -1;
-}
-#endif
-
 int
 _eXosip_insubscription_answer_3456xx (eXosip_notify_t * jn,
                                       eXosip_dialog_t * jd, int code)
@@ -739,3 +465,5 @@ _eXosip_insubscription_answer_3456xx (eXosip_notify_t * jn,
   __eXosip_wakeup ();
   return 0;
 }
+
+#endif
